// Smoother.cpp: Smoother NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Smoother.h"
#include "vtkImplicitFunction.h"
#include "Node.h"
#include "H1Error.h"
#include "LocalH1Error.h"
#include "EnergyMinimizer.h"
#include "LocalTangentError.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

Smoother::Smoother()
{
	T = NULL;
	center = NULL;
	m = NULL;
	valid = NULL;
}

Smoother::~Smoother()
{
	if(center != NULL)
		delete[] center;
	if(m != NULL)
		delete[] m;
	if(valid != NULL)
		delete[] valid;
}

void Smoother::LaplacianFlow(int iter, float dt)
{
	int vertex_N = mesh->vertex_N;
	//L is Lplacian vectors
	double (*L)[3] = new double[vertex_N][3];

	for(int i=0; i<iter; i++){
		for(int j=0; j<vertex_N; j++){
			//Boundary points are fixed
			if(mesh->isBound[j]){
				L[j][0] = L[j][1] = L[j][2] = 0;
			}	
			else{
				mesh->laplacian(j, L[j]);
			}
		}
		
		for(j=0; j<vertex_N; j++){
			float *p = mesh->vertex[j];
			//replace each vertex
			p[0] += (float)(dt*L[j][0]);
			p[1] += (float)(dt*L[j][1]);
			p[2] += (float)(dt*L[j][2]);
		}
	}
	delete[] L;
}

void Smoother::meanCurvatureFlow(int iter, float dt)
{
	int vertex_N = mesh->vertex_N;
	//Hn is mean curvature normals
	double (*Hn)[3] = new double[vertex_N][3];

	for(int i=0; i<iter; i++){
		for(int j=0; j<vertex_N; j++){
			//Boundary points are fixed
			if(mesh->isBound[j]){
				Hn[j][0] = Hn[j][1] = Hn[j][2] = 0;
			}
			else{
				//compute mean curvature normal
				mesh->meanCurvatureNormal(j, Hn[j]);
			}
		}
		
		for(j=0; j<vertex_N; j++){
			float *p = mesh->vertex[j];
			//replace each vertex
			p[0] -= (float)(dt*Hn[j][0]);
			p[1] -= (float)(dt*Hn[j][1]);
			p[2] -= (float)(dt*Hn[j][2]);
		}
	}
	delete[] Hn;
}

void Smoother::setMeshData(MeshData *mesh)
{
	this->mesh = mesh;

	if(center != NULL)
		delete[] center;
	center = NULL;

	if(m != NULL)
		delete[] m;
	m = NULL;

	if(valid != NULL)
		delete[] valid;
	valid = NULL;
}

void Smoother::GaussianNoise(float size)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;

	float noise;
	srand(100);
	for(int i=0; i<vertex_N; i++){
		double r1, r2, r;
		do{
			r1 = 2.0*((double)rand())/RAND_MAX - 1.0;
			r2 = 2.0*((double)rand())/RAND_MAX - 1.0;
			r = r1*r1 + r2*r2;
		}while(r >= 1.0 || r == 0.0);
		double f = sqrt(-2.0*log(r)/r);

		noise = (float)(size*r1*f);
		vertex[i][0] += normal[i][0]*noise;
		vertex[i][1] += normal[i][1]*noise;
		vertex[i][2] += normal[i][2]*noise;

		if(++i == vertex_N)
			return;
		else{
			noise = (float)(size*r2*f);		
			vertex[i][0] += normal[i][0]*noise;
			vertex[i][1] += normal[i][1]*noise;
			vertex[i][2] += normal[i][2]*noise;
		}
	}
}

void Smoother::computeNRingMeanCurvature(int n)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;

	if(T != NULL)
		delete[] T;

	T = new float[vertex_N][3];

	for(int i=0; i<vertex_N; i++){

		int **nei, *nei_N;
		mesh->getConsistentNRings(i, n, nei, nei_N);

		float total_w = 0;
		float total_Hn[3];
		total_Hn[0] = total_Hn[1] = total_Hn[2] = 0;

		for(int j=n-1; j<n; j++){

			if(nei_N[j] == 0 || mesh->isBound[nei[j][0]])
				continue;

			float A_j = 0;
			float Hn_j[3];
			Hn_j[0] = Hn_j[1] = Hn_j[2] = 0;

			for(int k=0; k<nei_N[j]; k+=j+1){
				int s = nei[j][k];
				int t = nei[j][(k+j+1)%nei_N[j]];

				//vectors of edges of triangles
				float PQ1[3], PQ2[3], QQ[3];
				MeshData::VEC(PQ1, vertex[i], vertex[s]);
				MeshData::VEC(PQ2, vertex[i], vertex[t]);
				MeshData::VEC(QQ, vertex[s], vertex[t]);

				//normal vector of triangle
				float normal_t[3];
				MeshData::CROSS(normal_t, PQ1, PQ2); 

				//area of triangle
				float A_k = (float)MeshData::AREA(vertex[i], vertex[s], vertex[t]);
		
				if(A_k != 0){
					A_j += A_k;

					float dot1 = (float)MeshData::DOT(PQ1, QQ);
					float dot2 = -(float)MeshData::DOT(PQ2,QQ);

					float cot1 = dot2/A_k;
					float cot2 = dot1/A_k;

					Hn_j[0] += cot1*PQ1[0] + cot2*PQ2[0];
					Hn_j[1] += cot1*PQ1[1] + cot2*PQ2[1];
					Hn_j[2] += cot1*PQ1[2] + cot2*PQ2[2];
				}
			}
			if(A_j != 0){
				Hn_j[0] /= A_j;
				Hn_j[1] /= A_j;
				Hn_j[2] /= A_j;

				total_w += 1.0f; ///A_j;

				total_Hn[0] += Hn_j[0]; ///A_j;
				total_Hn[1] += Hn_j[1]; ///A_j;
				total_Hn[2] += Hn_j[2]; ///A_j;
			}
		}
	
		for(j=0; j<n; j++)
			delete[] nei[j];
		delete[] nei;
		delete[] nei_N;

		if(total_w != 0){
			T[i][0] = total_Hn[0]/total_w;
			T[i][1] = total_Hn[1]/total_w;
			T[i][2] = total_Hn[2]/total_w;
		}
		else
			T[i][0] = T[i][1] = T[i][2] = 0;
	}
}

void Smoother::meanCurvatureFlow12Ring(int iter, float dt, float e)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	//Hn is mean curvature normals
	double (*Hn)[3] = new double[vertex_N][3];

	// index of 2-ring
	int **nei2, *nei2_N;
	nei2 = (int **)new int[vertex_N];
	nei2_N = new int[vertex_N];
	for(int i=0; i<vertex_N; i++){
		int **nei, *nei_N;
		mesh->getConsistentNRings(i, 2, nei, nei_N);

		nei2_N[i] = nei_N[1];
		nei2[i] = nei[1];

		delete[] nei[0];
		delete[] nei;
		delete[] nei_N;
	}

	for(i=0; i<iter; i++){
		for(int j=0; j<vertex_N; j++){
			//Boundary points are fixed
			if(mesh->isBound[j]){
				Hn[j][0] = Hn[j][1] = Hn[j][2] = 0;
			}
			else{
				//compute mean curvature normal
				mesh->meanCurvatureNormal(j, Hn[j]);
			
				if(nei2_N[j] == 0 || mesh->isBound[nei2[j][0]])
					continue;

				float A_2 = 0;
				float Hn_2[3];
				Hn_2[0] = Hn_2[1] = Hn_2[2] = 0;

				for(int k=0; k<nei2_N[j]; k += 2){
					int s = nei2[j][k];
					int t = nei2[j][(k+2)%nei2_N[j]];

					//vectors of edges of triangles
					float PQ1[3], PQ2[3], QQ[3];
					MeshData::VEC(PQ1, vertex[j], vertex[s]);
					MeshData::VEC(PQ2, vertex[j], vertex[t]);
					MeshData::VEC(QQ, vertex[s], vertex[t]);

					//normal vector of triangle
					float normal_t[3];
					MeshData::CROSS(normal_t, PQ1, PQ2); 

					//area of triangle
					float A_k = (float)MeshData::AREA(vertex[j], vertex[s], vertex[t]);
		
					if(A_k != 0){
						A_2 += A_k;

						float dot1 = (float)MeshData::DOT(PQ1, QQ);
						float dot2 = -(float)MeshData::DOT(PQ2,QQ);

						float cot1 = dot2/A_k;
						float cot2 = dot1/A_k;

						Hn_2[0] += cot1*PQ1[0] + cot2*PQ2[0];
						Hn_2[1] += cot1*PQ1[1] + cot2*PQ2[1];
						Hn_2[2] += cot1*PQ1[2] + cot2*PQ2[2];
					}
				}
				if(A_2 != 0){
					Hn_2[0] /= A_2;
					Hn_2[1] /= A_2;
					Hn_2[2] /= A_2;
				}


				float H2 = MeshData::DOT(Hn_2, Hn_2);
				float H12 = MeshData::DOT(Hn[j], Hn_2);
			
				if(H12 < 0){
				}
				else if(H12 > (1.0f+e)*H2){
					Hn[j][0] *= (1.0f-H2/H12);
					Hn[j][1] *= (1.0f-H2/H12);
					Hn[j][2] *= (1.0f-H2/H12);
				}
				//else if(H12 < (1.0f-e)*H2){
					//Hn[j][0] *= (H2/H12-1.0f);
					//Hn[j][1] *= (H2/H12-1.0f);
					//Hn[j][2] *= (H2/H12-1.0f);
				//}
				else{
					Hn[j][0] = Hn[j][1] = Hn[j][2] = 0;
				}
			}
		}
		
		for(j=0; j<vertex_N; j++){
			float *p = vertex[j];
			//replace each vertex
			p[0] -= dt*Hn[j][0];
			p[1] -= dt*Hn[j][1];
			p[2] -= dt*Hn[j][2];
		}
	}
	delete[] Hn;

	for(i=0; i<vertex_N; i++){
		delete[] nei2[i];
	}
	delete[] nei2;
	delete[] nei2_N;
}

void Smoother::correctImplicitSurface(vtkImplicitFunction *func, int iter, int off_iter,
									  float T, int projection, 
									  int uniform, int keep, BOOL err, int from)
{
	int i;

	/*
	int n = mesh->vertex_N;

	H1Error h1;
	h1.f = func;
	h1.mesh = mesh;

	EnergyMinimizer mini;
	mini.energy = &h1;
	
	int iter1;
	float fret;
	float *p = new float[3*n];
	for(i=0; i<n; i++){
		p[3*i] = mesh->vertex[i][0];
		p[3*i+1] = mesh->vertex[i][1];
		p[3*i+2] = mesh->vertex[i][2];
	}

	mini.frprmn(p, 3*n, 0.01f, &iter1, &fret);

	for(i=0; i<n; i++){
		mesh->vertex[i][0] = p[3*i];
		mesh->vertex[i][1] = p[3*i+1];
		mesh->vertex[i][2] = p[3*i+2];
	}
	return;
	*/

	FILE *v_file, *f_file;
	float c;
	if(err){	
		v_file = fopen("vertex_err.dat","w");
		fprintf(v_file, "{");
		float max = 0;
		for(int i=0; i<mesh->vertex_N; i++){
			float f = func->EvaluateFunction(mesh->vertex[i]);
			if(max < f*f)
				max = f*f;
		}
		//c = 1.0f/max;
		c = 1;
		fprintf(v_file, "%f", c*computeVertexErr(func));

		f_file = fopen("face_err.dat","w");
		fprintf(f_file, "{");
		fprintf(f_file, "%f", computeFaceNormalErr(func));
	}

	//setNormalAsGrad(func);
	//return;

	//int i;
	
	// fix outside of the solid
	/*
	mesh->computeFaceNormal();
	mesh->computeNormal();
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	for(i=0; i<vertex_N; i++)
		if(func->EvaluateFunction(vertex[i]) <= 0)
			mesh->isBound[i] = true;
	this->detectSolidBound();
	*/

	//mesh->computeFaceNormal();
	//this->setFixedVertex(T, func);
	for(i=0; i<iter + off_iter; i++){
		/*
		if(projection == 1){
			setNormalAsGrad(func);
			//smoothNormal(1.5);
			
			//moveNormalWithF(1000, func);

			//if(i < iter/2)
				moveNormal(from);
			//else
				//moveNormalD(10, 0.1);
		}
		else if(projection == 0){
			mesh->computeFaceNormal();
			mesh->computeNormal();
			smoothNormal(T);
			moveNormal(from);
		}

		if(uniform == 0 && i < iter){
			//mesh->computeFaceNormal();
			mesh->computeNormal();
			//setVertexNormalAsGrad(func);
			//tangentLapW(0.1, T);
			tangentLap(0.1, from);
		}
		else if(uniform == 1){
			//mesh->computeFaceNormal();
			mesh->computeNormal();
			//tangentLap(0.1);
			//setVertexNormalAsGrad(func);
			tangentLap(1.0f - (float)i/iter, from); //, T);
			//tangentLap(0.1, from);
		}
		*/	
			
		if(keep == 0){
			//mesh->computeFaceNormal();
			//mesh->computeNormal();
			//if(i < iter/2)
				keepOnSurface(func, from);
			//else if(i < iter-1)
				//keepOnSurfaceT(func, 0.01f);
			//keepOnSurfaceUT(func, 0.01f, 0.1f, T);
			//keepOnSurfaceUTN(func, 0.01f, 0.1f, 0.0001f, T);
		}
		if(err){
			fprintf(v_file, ",%f", c*computeVertexErr(func));
			fprintf(f_file, ",%f", computeFaceNormalErr(func));
		}
	}
	//mesh->computeFaceNormal();
	//this->setFixedVertex(T, func);

	if(err){
		fprintf(v_file, "}");
		fprintf(f_file, "}");
		fclose(v_file);
		fclose(f_file);
	}
}

void Smoother::smoothNormal(float T)
{
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal_f;
	int (*face)[3] = mesh->face;
	int face_N = mesh->face_N;
	float (*tmp_normal)[3] = new float[face_N][3];
	int (*ad_face)[3] = mesh->face_link_E;
	for(int i=0; i<face_N; i++){
		float total_w = 0;
		tmp_normal[i][0] = 0;
		tmp_normal[i][1] = 0;
		tmp_normal[i][2] = 0;
		float c[3];
		c[0] = (vertex[face[i][0]][0] + vertex[face[i][1]][0] + vertex[face[i][2]][0])/3.0f;
		c[1] = (vertex[face[i][0]][1] + vertex[face[i][1]][1] + vertex[face[i][2]][1])/3.0f;
		c[2] = (vertex[face[i][0]][2] + vertex[face[i][1]][2] + vertex[face[i][2]][2])/3.0f;
		for(int j=0; j<3; j++){
			int pair = ad_face[i][j];
			if(pair < 0)
				continue;
		
			float c1[3];
			c1[0] = (vertex[face[pair][0]][0] + vertex[face[pair][1]][0] + vertex[face[pair][2]][0])/3.0f;
			c1[1] = (vertex[face[pair][0]][1] + vertex[face[pair][1]][1] + vertex[face[pair][2]][1])/3.0f;
			c1[2] = (vertex[face[pair][0]][2] + vertex[face[pair][1]][2] + vertex[face[pair][2]][2])/3.0f;
			float d = MeshData::DIST(c,c1);

			float area = MeshData::AREA(vertex[face[pair][0]], vertex[face[pair][1]], vertex[face[pair][2]]);

			float k = acos(MeshData::DOT(normal[i], normal[pair]))/d;
			float w = area*exp(-(k/T)*(k/T));
			total_w += w;
			tmp_normal[i][0] += w*normal[pair][0];
			tmp_normal[i][1] += w*normal[pair][1];
			tmp_normal[i][2] += w*normal[pair][2];
		}
		if(total_w != 0){
			tmp_normal[i][0] /= total_w;
			tmp_normal[i][1] /= total_w;
			tmp_normal[i][2] /= total_w;
		}
		else{
			tmp_normal[i][0] = normal[i][0];
			tmp_normal[i][1] = normal[i][1];
			tmp_normal[i][2] = normal[i][2];
		}

		float len = MeshData::LENGTH(tmp_normal[i]);
		tmp_normal[i][0] /= len;
		tmp_normal[i][1] /= len;
		tmp_normal[i][2] /= len;
	}
	delete[] normal;
	mesh->normal_f = tmp_normal;
}

void Smoother::moveNormal(int from)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;
	double (*dp)[3] = new double[vertex_N][3];
	int **link_f = mesh->vertex_link_f;
	int **link_v = mesh->vertex_link_v;
	int *degree_f = mesh->degree_f;
	int *degree_v = mesh->degree_v;

	for(int i=from; i<vertex_N; i++){
		//if(mesh->isFixed[i])
			//continue;

		if(mesh->isBound[i] == NON_MANIFOLD){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			//continue;

			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.001)
				continue;

			int f1 = link_f[i][0];
			int f2 = link_f[i][degree_f[i]-1];

			float n1[3], n2[3];
			n1[0] = normal_f[f1][0];
			n1[1] = normal_f[f1][1];
			n1[2] = normal_f[f1][2];

			n2[0] = normal_f[f2][0];
			n2[1] = normal_f[f2][1];
			n2[2] = normal_f[f2][2];

			double dot1 = MeshData::DOT(n1, n)/len;
			double dot2 = MeshData::DOT(n2, n)/len;

			n1[0] -= dot1*n[0]/len;
			n1[1] -= dot1*n[1]/len;
			n1[2] -= dot1*n[2]/len;
			double len1 = MeshData::LENGTH(n1);
			if((float)len1 == 0)
				continue;
			n1[0] /= len1;
			n1[1] /= len1;
			n1[2] /= len1;

			n2[0] -= dot2*n[0]/len;
			n2[1] -= dot2*n[1]/len;
			n2[2] -= dot2*n[2]/len;
			double len2 = MeshData::LENGTH(n2);
			if((float)len2 == 0)
				continue;
			n2[0] /= len2;
			n2[1] /= len2;
			n2[2] /= len2;

			dot1 = 0.5*MeshData::DOT(v1, n1);
			dot2 = 0.5*MeshData::DOT(v2, n2);

			len1 = MeshData::LENGTH(v1);
			len2 = MeshData::LENGTH(v2);
			len = len1 + len2;

			if((float)len != 0){
				dp[i][0] = (len1*dot1*n1[0] + len2*dot2*n2[0])/len;
				dp[i][1] = (len1*dot1*n1[1] + len2*dot2*n2[1])/len;
				dp[i][2] = (len1*dot1*n1[2] + len2*dot2*n2[2])/len;
			}
			continue;
		}

		int *nei, *nei_v, nei_N;
		mesh->getConsistent1Ring(i, nei_v, nei, nei_N);
		if(nei_N == 0){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		double total_w = 0;

		double pro[3];
		pro[0] = pro[1] = pro[2] = 0;

		BOOL flip_flag = false;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = face[f][0];
			int i1 = face[f][1];
			int i2 = face[f][2];

			double o[3];
			o[0] = (vertex[i0][0] + vertex[i1][0] + vertex[i2][0])/3.0;
			o[1] = (vertex[i0][1] + vertex[i1][1] + vertex[i2][1])/3.0;
			o[2] = (vertex[i0][2] + vertex[i1][2] + vertex[i2][2])/3.0;

			double d[3];
			MeshData::VEC(d, o, vertex[i]);
			double inner = MeshData::DOT(normal_f[f], d);

			/*
			float v1[3], v2[3];
			if(i0 == i){
				MeshData::VEC(v1, vertex[i0], vertex[i1]);
				MeshData::VEC(v2, vertex[i0], vertex[i2]);
			}
			else if(i1 == i){
				MeshData::VEC(v1, vertex[i1], vertex[i0]);
				MeshData::VEC(v2, vertex[i1], vertex[i2]);
			}
			else{
				MeshData::VEC(v1, vertex[i2], vertex[i0]);
				MeshData::VEC(v2, vertex[i2], vertex[i1]);
			}
			double len1 = MeshData::LENGTH(v1);
			double len2 = MeshData::LENGTH(v2);

			float w;
			if((float)len1 == 0 || (float)len2 == 0){
				w = 0;
			}
			else{
				double dot = MeshData::DOT(v1, v2)/(len1*len2);
				if(dot > 1.0)
					dot = 1;
				else if(dot < -1.0)
					dot = -1;
				w = acos(dot);
			}*/
			double w = MeshData::AREA(vertex[i0], vertex[i1], vertex[i2]);

			total_w += w;

			pro[0] -= w*inner*normal_f[f][0];
			pro[1] -= w*inner*normal_f[f][1];
			pro[2] -= w*inner*normal_f[f][2];

			//float n[3];
			//float v1[3];
			//float v2[3];
			//MeshData::VEC(v1, vertex[i0], vertex[i1]);
			//MeshData::VEC(v2, vertex[i0], vertex[i2]);
			//MeshData::CROSS(n, v1, v2);
			//if(MeshData::DOT(normal_f[f], n) > 0.7){
				//flip_flag = true;
				//break;
				//TRACE("FLIP\n");
			//}
		}
		if(flip_flag){
			//mesh->laplacian(i, dp[i]);
			/*
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			for(int j=0; j<nei_N; j++){
				dp[i][0] += vertex[nei_v[j]][0] - vertex[i][0];
				dp[i][1] += vertex[nei_v[j]][1] - vertex[i][0];
				dp[i][2] += vertex[nei_v[j]][2] - vertex[i][0];
			}
			dp[i][0] /= (float)nei_N;
			dp[i][1] /= (float)nei_N;
			dp[i][2] /= (float)nei_N;*/
		}
		else if(total_w != 0){
			dp[i][0] = pro[0]/total_w;
			dp[i][1] = pro[1]/total_w;
			dp[i][2] = pro[2]/total_w;
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}

	//mesh->computeFaceNormal();
	//mesh->computeNormal();
	for(i=from; i<vertex_N; i++){
		//if(mesh->isFixed[i])
			//continue;

		//if(MeshData::DOT(dp[i], mesh->normal[i]) > 0 && !isSolidBound[i]){
			vertex[i][0] += dp[i][0];
			vertex[i][1] += dp[i][1];
			vertex[i][2] += dp[i][2];
		//}
	}
	delete[] dp;
}

void Smoother::keepOnSurface(vtkImplicitFunction *func, int from)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;
	int **link_v = mesh->vertex_link_v;
	int **link_f = mesh->vertex_link_f;
	int *degree_f = mesh->degree_f;
	int *degree_v = mesh->degree_v;
	int i;
	float (*g)[3] = new float[vertex_N][3];
	
	float max_V = 0;
	for(i=from; i<vertex_N; i++){
		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		func->EvaluateGradient(vertex[i], g[i]);
		float f = func->EvaluateFunction(vertex[i]);
		float len = MeshData::LENGTH(g[i]);

		int deg = degree_f[i];
		int* l = link_f[i];
		double a = 0;
		for(int j=0; j<deg; j++)
			a += mesh->faceArea(l[j]);
		

		float V = a*fabs(f*len);
		if(V > max_V)
			max_V = V;

		g[i][0] *= a*f;
		g[i][1] *= a*f;
		g[i][2] *= a*f;
	}
	if(max_V == 0)
		return;
	//float dt = 0.1f/max_V;
	float dt = 0.01f/max_V;
	if(dt < 0.000000001)
		dt = 0.0001;

	for(i=from; i<vertex_N; i++){
		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		if(mesh->isBound[i]){
			//continue;

			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.0001)
				continue;
			
			double dot = MeshData::DOT(n, g[i])/len;
			
			vertex[i][0] -= (g[i][0] - dot*n[0]/len)*dt;
			vertex[i][1] -= (g[i][1] - dot*n[1]/len)*dt;
			vertex[i][2] -= (g[i][2] - dot*n[2]/len)*dt;
			
			continue;
		}
		/*
		if(isSolidBound[i]){
			float v[3];
			v[0] = -f*g[0]*dt;
			v[1] = -f*g[1]*dt;
			v[2] = -f*g[2]*dt;
			float inner = MeshData::DOT(originalNormal[i], v);
			vertex[i][0] += v[0] - inner*originalNormal[i][0];
			vertex[i][1] += v[1] - inner*originalNormal[i][1];
			vertex[i][2] += v[2] - inner*originalNormal[i][2];
		}
		else if(MeshData::DOT(normal[i], g) < 0){*/
			vertex[i][0] -= g[i][0]*dt;
			vertex[i][1] -= g[i][1]*dt;
			vertex[i][2] -= g[i][2]*dt;
		/*}
		else{
			float len = MeshData::LENGTH(g);
			vertex[i][0] += fabs(f)*len*dt*normal[i][0];
			vertex[i][1] += fabs(f)*len*dt*normal[i][1];
			vertex[i][2] += fabs(f)*len*dt*normal[i][2];
		}*/
	}
	delete[] g;
}

void Smoother::tangentLap(float dt, int from)
{
	int vertex_N = mesh->vertex_N;
	double (*dp)[3] = new double[vertex_N][3];
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;
	int **link_v = mesh->vertex_link_v;
	int *degree_v = mesh->degree_v;

	for(int i=from; i<vertex_N; i++){
		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			//continue;
			
			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);
			double len1 = MeshData::LENGTH(v1);
			double len2 = MeshData::LENGTH(v2);
			double n[3];
			n[0] = len2*v1[0] + len1*v2[0];
			n[1] = len2*v1[1] + len1*v2[1];
			n[2] = len2*v1[2] + len1*v2[2];
			double len = MeshData::LENGTH(n);

			if((float)len < 0.001){
				dp[i][0] = dp[i][1] = dp[i][2] = 0;
				continue;
			}

			double v[3];
			v[0] = 0.5*(v1[0] + v2[0]);
			v[1] = 0.5*(v1[1] + v2[1]);
			v[2] = 0.5*(v1[2] + v2[2]);

			double dot = MeshData::DOT(v, n)/len;

			dp[i][0] = v[0] - dot*n[0]/len;
			dp[i][1] = v[1] - dot*n[1]/len;
			dp[i][2] = v[2] - dot*n[2]/len;
			
			continue;
		}
		double L[3];
		mesh->laplacian(i, L);
		float inner = MeshData::DOT(normal[i], L);
		dp[i][0] = L[0] - inner*normal[i][0];
		dp[i][1] = L[1] - inner*normal[i][1];
		dp[i][2] = L[2] - inner*normal[i][2];

		//dp[i][0] = L[0] - inner*normal[i][0] + 0.1*normal[i][0];
		//dp[i][1] = L[1] - inner*normal[i][1] + 0.1*normal[i][1];
		//dp[i][2] = L[2] - inner*normal[i][2] + 0.1*normal[i][2];
	}
		
	for(i=from; i<vertex_N; i++){
		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		//if(isSolidBound[i])
			//continue;

		vertex[i][0] += dt*dp[i][0];
		vertex[i][1] += dt*dp[i][1];
		vertex[i][2] += dt*dp[i][2];
	}

	delete[] dp;
}

void Smoother::principalFlow(float dt, float T)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;

	mesh->computeFaceNormal();
	mesh->computeNormal();
	mesh->computePrincipal();

	double (*k_max) = mesh->k_max;
	double (*k_min) = mesh->k_min;
	float (*normal)[3] = mesh->normal;

	float (*dp)[3] = new float[vertex_N][3];

	for(int i=0; i<vertex_N; i++){
		float w = 0;
		float mean = 0.5f*(k_max[i] + k_min[i]);
		if(fabs(k_max[i]) <= T && fabs(k_min[i]) <= T){
			w = mean;
		}
		else if(fabs(k_max[i]) > T && fabs(k_min[i]) > T && k_max[i]*k_min[i] > 0){
			w = 0;
		}
		else if(min(min(fabs(k_max[i]), fabs(k_min[i])),fabs(mean)) == fabs(k_max[i])){
			w = k_max[i];
		}
		else if(min(min(fabs(k_max[i]), fabs(k_min[i])),fabs(mean)) == fabs(k_min[i])){
			w = k_min[i];
		}
		else if(min(min(fabs(k_max[i]), fabs(k_min[i])),fabs(mean)) == fabs(mean)){
			w = mean;
		}
		dp[i][0] = w*normal[i][0];
		dp[i][1] = w*normal[i][1];
		dp[i][2] = w*normal[i][2];
	}

	for(i=0; i<vertex_N; i++){
		vertex[i][0] -= dt*dp[i][0];
		vertex[i][1] -= dt*dp[i][1];
		vertex[i][2] -= dt*dp[i][2];
	}
	delete[] dp;
}

void Smoother::setNormalAsGrad(vtkImplicitFunction* func)
{
	int face_N = mesh->face_N;
	int (*face)[3] = mesh->face;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal_f;
	if(normal == NULL)
		normal = mesh->normal_f = new float[face_N][3];

	for(int i=0; i<face_N; i++){
		int* fi = face[i];
		if(fi[0] < 0)
			continue;
		float c[3];
		c[0] = (vertex[fi[0]][0] + vertex[fi[1]][0] + vertex[fi[2]][0])/3.0f;
		c[1] = (vertex[fi[0]][1] + vertex[fi[1]][1] + vertex[fi[2]][1])/3.0f;
		c[2] = (vertex[fi[0]][2] + vertex[fi[1]][2] + vertex[fi[2]][2])/3.0f;
		
		float g[3];
		func->EvaluateGradient(c, g);
		double len = MeshData::LENGTH(g);
		if((float)len > 0.000001){	
			normal[i][0] = -(float)g[0]/len;
			normal[i][1] = -(float)g[1]/len;
			normal[i][2] = -(float)g[2]/len;
		}
		else{
			normal[i][0] = normal[i][1] = normal[i][2] = 0;
		}
	}
}

void Smoother::tangentLapW(float dt, float T)
{
	int vertex_N = mesh->vertex_N;
	float (*dp)[3] = new float[vertex_N][3];
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;

	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		float L[3];
		L[0] = L[1] = L[2] = 0;
		float total_w = 0;
		int *nei, nei_N;
		mesh->getConsistent1Ring(i, nei, nei_N);
		for(int j=0; j<nei_N; j++){
			float v[3];
			MeshData::VEC(v, vertex[i], vertex[nei[j]]);
			float k = 2.0f*MeshData::DOT(normal[i], v)/(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
	
			//float k = acos(MeshData::DOT(normal[i], normal[nei[j]]))/MeshData::LENGTH(v);

			float w;
			//if(k > T)
				//w = 0;
			//else
				w = exp(-k*k/(T*T));
			total_w += w;
			L[0] += w*v[0];
			L[1] += w*v[1];
			L[2] += w*v[2];
		}

		if(total_w != 0){
			L[0] /= total_w;
			L[1] /= total_w;
			L[2] /= total_w;

			float inner = MeshData::DOT(normal[i], L);
			dp[i][0] = L[0] - inner*normal[i][0];
			dp[i][1] = L[1] - inner*normal[i][1];
			dp[i][2] = L[2] - inner*normal[i][2];
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}
		
	for(i=0; i<vertex_N; i++){
		vertex[i][0] += dt*dp[i][0];
		vertex[i][1] += dt*dp[i][1];
		vertex[i][2] += dt*dp[i][2];
	}

	delete[] dp;
}

float Smoother::computeVertexErr(vtkImplicitFunction *func)
{
	float err = 0;
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float total_area = 0;
	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i])
			continue;
		int *nei, nei_N;
		float area = 0;
		mesh->getConsistent1Ring(i, nei, nei_N);
		for(int j=0; j<nei_N; j++)
			area += MeshData::AREA(vertex[i], vertex[nei[j]], vertex[nei[(j+1)%nei_N]]);
		float f = func->EvaluateFunction(vertex[i]);
		float g[3];
		func->EvaluateGradient(vertex[i], g);
		double len = g[0]*g[0] + g[1]*g[1] + g[2]*g[2];
		if((float)len == 0)
			continue;
		err += area*f*f/len;
		total_area += area; 
	}
	return sqrt(err/(3.0*total_area));
}

float Smoother::computeFaceNormalErr(vtkImplicitFunction *func)
{
	float err = 0;
	int face_N = mesh->face_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	BOOL *isBound = mesh->isBound;
	float total_area = 0;

	for(int i=0; i<face_N; i++){
		int* fi = face[i];
		//if(isBound[fi[0]] || isBound[fi[1]] || isBound[fi[2]])
			//continue;

		float c[3];
		c[0] = (vertex[fi[0]][0] + vertex[fi[1]][0] + vertex[fi[2]][0])/3.0f;
		c[1] = (vertex[fi[0]][1] + vertex[fi[1]][1] + vertex[fi[2]][1])/3.0f;
		c[2] = (vertex[fi[0]][2] + vertex[fi[1]][2] + vertex[fi[2]][2])/3.0f;
		float g[3];
		func->EvaluateGradient(c, g);
		float len1 = MeshData::LENGTH(g);

		float v1[3], v2[3], n[3];
		MeshData::VEC(v1, vertex[fi[0]], vertex[fi[1]]);
		MeshData::VEC(v2, vertex[fi[0]], vertex[fi[2]]);
		MeshData::CROSS(n, v1, v2);
		float len2 = MeshData::LENGTH(n);
		
		if(len1*len2 == 0)
			continue;

		float dot = MeshData::DOT(g,n)/(len1*len2);

		err += len2*(1.0f+dot);
		total_area += len2;
	}
	return sqrt(err/total_area);
}

void Smoother::setVertexNormalAsGrad(vtkImplicitFunction *func)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;
	if(normal == NULL)
		normal = mesh->normal = new float[vertex_N][3];

	for(int i=0; i<vertex_N; i++){
		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float len = MeshData::LENGTH(g);
	
		normal[i][0] = g[0]/len;
		normal[i][1] = g[1]/len;
		normal[i][2] = g[2]/len;
	}
}


void Smoother::correctImplicitSurface2(vtkImplicitFunction *func, int iter, float T, int projection, int uniform, int keep, BOOL err)
{
	FILE *v_file, *f_file;
	if(err){	
		v_file = fopen("vertex_err.dat","w");
		fprintf(v_file, "{");
		fprintf(v_file, "%f", computeVertexErr(func));

		f_file = fopen("face_err.dat","w");
		fprintf(f_file, "{");
		fprintf(f_file, "%f", computeFaceNormalErr(func));
	}
	
	//setNormalAsGrad(func);
	//return;

	int i;
	
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	/*
	for(i=0; i<vertex_N; i++)
		if(func->EvaluateFunction(vertex[i]) < 0)
			mesh->isBound[i] = true;
	*/

	float (*F)[3] = new float[vertex_N][3];

	for(i=0; i<iter; i++){
		for(int j=0; j<vertex_N; j++)
			F[j][0] = F[j][1] = F[j][2] = 0;

		if(projection == 1){
			setNormalAsGrad(func);
			moveNormal2(F);
		}
		else if(projection == 0){
			mesh->computeFaceNormal();
			mesh->computeNormal();
			smoothNormal(T);
			moveNormal2(F);
		}

		if(uniform == 0 && i < iter/2){
			mesh->computeFaceNormal();
			mesh->computeNormal();
			//setVertexNormalAsGrad(func);
			//tangentLapW(0.1, T);
			tangentLap2(0.1, F);
		}
		else if(uniform == 1){
			mesh->computeFaceNormal();
			mesh->computeNormal();
			tangentLap2(0.1, F);
			//setVertexNormalAsGrad(func);
			//tangentLapW(0.1, T);
		}
			
		if(keep == 0)
			keepOnSurface2(func, F);

		for(j=0; j<vertex_N; j++){
			vertex[j][0] += F[j][0];
			vertex[j][1] += F[j][1];
			vertex[j][2] += F[j][2];
		}
		if(err){
			fprintf(v_file, ",%f", computeVertexErr(func));
			fprintf(f_file, ",%f", computeFaceNormalErr(func));
		}
	}
	delete[] F;

	if(err){
		fprintf(v_file, "}", computeVertexErr(func));
		fprintf(f_file, "}", computeFaceNormalErr(func));
		fclose(v_file);
		fclose(f_file);
	}
}


void Smoother::moveNormal2(float (*F)[3])
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;

	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i]){
			continue;
		}

		int *nei, *nei_v, nei_N;
		mesh->getConsistent1Ring(i, nei_v, nei, nei_N);
		if(nei_N == 0){
			continue;
		}

		float total_w = 0;

		float pro[3];
		pro[0] = pro[1] = pro[2] = 0;

		BOOL flip_flag = false;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = face[f][0];
			int i1 = face[f][1];
			int i2 = face[f][2];

			float o[3];
			o[0] = (vertex[i0][0] + vertex[i1][0] + vertex[i2][0])/3.0;
			o[1] = (vertex[i0][1] + vertex[i1][1] + vertex[i2][1])/3.0;
			o[2] = (vertex[i0][2] + vertex[i1][2] + vertex[i2][2])/3.0;

			float d[3];
			MeshData::VEC(d, o, vertex[i]);
			float inner = MeshData::DOT(normal_f[f], d);

			float w = MeshData::AREA(vertex[i0], vertex[i1], vertex[i2]);
			total_w += w;

			pro[0] -= w*inner*normal_f[f][0];
			pro[1] -= w*inner*normal_f[f][1];
			pro[2] -= w*inner*normal_f[f][2];

			float n[3];
			float v1[3];
			float v2[3];
			MeshData::VEC(v1, vertex[i0], vertex[i1]);
			MeshData::VEC(v2, vertex[i0], vertex[i2]);
			MeshData::CROSS(n, v1, v2);
			//if(MeshData::DOT(normal_f[f], n) > 0.7){
				//flip_flag = true;
				//break;
				//TRACE("FLIP\n");
			//}
		}
		if(flip_flag){
			//mesh->laplacian(i, dp[i]);
			/*
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			for(int j=0; j<nei_N; j++){
				dp[i][0] += vertex[nei_v[j]][0] - vertex[i][0];
				dp[i][1] += vertex[nei_v[j]][1] - vertex[i][0];
				dp[i][2] += vertex[nei_v[j]][2] - vertex[i][0];
			}
			dp[i][0] /= (float)nei_N;
			dp[i][1] /= (float)nei_N;
			dp[i][2] /= (float)nei_N;*/
		}
		else if(total_w != 0){
			F[i][0] += pro[0]/total_w;
			F[i][1] += pro[1]/total_w;
			F[i][2] += pro[2]/total_w;
		}
	}
}

void Smoother::keepOnSurface2(vtkImplicitFunction *func, float (*F)[3])
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int i;
	
	float max_V = 0;
	for(i=0; i<vertex_N; i++){
		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float f = func->EvaluateFunction(vertex[i]);
		float len = MeshData::LENGTH(g);
		float V = fabs(f*len);
		//TRACE("%f\n", V);
		if(V > max_V)
			max_V = V;
	}
	
	//float dt = 0.1f/max_V;
	float dt = 0.1f/max_V;
	//if(dt < 0.000000001)
		//dt = 0.0001;

	for(i=0; i<vertex_N; i++){
		if(mesh->isBound[i])
			continue;

		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float f = func->EvaluateFunction(vertex[i]);
		F[i][0] -= f*g[0]*dt;
		F[i][1] -= f*g[1]*dt;
		F[i][2] -= f*g[2]*dt;
	}
}

void Smoother::tangentLap2(float dt, float (*F)[3])
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;

	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i]){
			continue;
		}
		double L[3];
		mesh->laplacian(i, L);
		float inner = MeshData::DOT(normal[i], L);
		F[i][0] += L[0] - inner*normal[i][0];
		F[i][1] += L[1] - inner*normal[i][1];
		F[i][2] += L[2] - inner*normal[i][2];

		//dp[i][0] = L[0] - inner*normal[i][0] + 0.1*normal[i][0];
		//dp[i][1] = L[1] - inner*normal[i][1] + 0.1*normal[i][1];
		//dp[i][2] = L[2] - inner*normal[i][2] + 0.1*normal[i][2];
	}
}

void Smoother::detectSolidBound()
{
	int vertex_N = mesh->vertex_N;
	int *isBound = mesh->isBound;
	
	isSolidBound = new BOOL[vertex_N];
	originalNormal = new float[vertex_N][3];

	for(int i=0; i<vertex_N; i++){
		isSolidBound[i] = false;

		if(isBound[i])
			continue;

		int *nei, nei_N;
		mesh->get1Ring(i, nei, nei_N);
		
		if(nei_N == 0)
			continue;

		for(int j=0; j<nei_N; j++)
			if(isBound[nei[j]] != 0)
				isSolidBound[i] = true;

		delete[] nei;

		if(isSolidBound[i]){
			originalNormal[i][0] = mesh->normal[i][0];
			originalNormal[i][1] = mesh->normal[i][1];
			originalNormal[i][2] = mesh->normal[i][2];
		}
	}
}

void Smoother::setFixedVertex(float T, vtkImplicitFunction* func)
{
	int vertex_N = mesh->vertex_N;
	bool* isFixed = mesh->isFixed;
	int **link_v = mesh->vertex_link_v;
	int **link_f = mesh->vertex_link_f;
	int *degree = mesh->degree_f;
	float (*normal)[3] = mesh->normal_f;
	float (*vertex)[3] = mesh->vertex;

	for(int i=0; i<vertex_N; i++){
		if(isFixed[i])
			continue;
		double err = 0;
		double total_area = 0;
		for(int j=0; j<degree[i]; j++){
			int f = link_f[i][j];
			double a = mesh->faceArea(f);
			total_area += a;
			float c[3];
			mesh->faceCenter(c, f);
			float g[3];
			func->EvaluateGradient(c, g);
			double len_g = MeshData::LENGTH(g);
			err += a*(1.0 - fabs(MeshData::DOT(normal[f], g)/len_g));
		}
		float ver = func->EvaluateFunction(vertex[i]);
		err = sqrt(err/total_area + ver*ver);
		if(err <= T)
			isFixed[i] = true;
		//else
			//isFixed[i] = false;
	}
}

void Smoother::keepOnSurfaceCenter(vtkImplicitFunction *func)
{
	int face_N = mesh->face_N;
	float (*center)[3] = mesh->center;
	int i;
	
	float max_V = 0;
	for(i=0; i<face_N; i++){
		float g[3];
		func->EvaluateGradient(center[i], g);
		float f = func->EvaluateFunction(center[i]);
		float len = MeshData::LENGTH(g);
		float V = fabs(f*len);
		if(V > max_V)
			max_V = V;
	}
	if(max_V == 0)
		return;
	float dt = 0.01f/max_V;

	for(i=0; i<face_N; i++){
		float g[3];
		func->EvaluateGradient(center[i], g);
		float f = func->EvaluateFunction(center[i]);

		center[i][0] -= f*g[0]*dt;
		center[i][1] -= f*g[1]*dt;
		center[i][2] -= f*g[2]*dt;
	}
}

void Smoother::tangentLapCenter(float dt)
{
	int face_N = mesh->face_N;
	float (*normal)[3] = mesh->normal_f;
	int (*link)[3] = mesh->face_link_E;
	double (*dp)[3] = new double[face_N][3];
	float (*center)[3] = mesh->center;

	for(int i=0; i<face_N; i++){
		dp[i][0] = dp[i][1] = dp[i][2] = 0;
		int valid = 0;
		for(int j=0; j<3; j++){
			int l = link[i][j];
			if(l < 0)
				continue;
			valid++;
			dp[i][0] += center[l][0];
			dp[i][1] += center[l][1];
			dp[i][2] += center[l][2];
		}
		if(valid != 0){
			dp[i][0] = dp[i][0]/valid - center[i][0];
			dp[i][1] = dp[i][1]/valid - center[i][1];
			dp[i][2] = dp[i][2]/valid - center[i][2];

			double dot = MeshData::DOT(normal[i], dp[i]);
			dp[i][0] = dp[i][0] - dot*normal[i][0];
			dp[i][1] = dp[i][1] - dot*normal[i][1];
			dp[i][2] = dp[i][2] - dot*normal[i][2];
		}
	}

	for(i=0; i<face_N; i++){
		center[i][0] += dt*dp[i][0];
		center[i][1] += dt*dp[i][1];
		center[i][2] += dt*dp[i][2];
	}
	delete[] dp;
}

void Smoother::correntImplicitCenter(vtkImplicitFunction *func, int iter)
{
	//mesh->computeFaceNormal();
	mesh->computeCenter();
	for(int i=0; i<iter; i++){
		//mesh->computeVertexFromCenter();
		keepOnSurfaceCenter(func);
		//setNormalAsGraCeneter(func);
		//tangentLapCenter(0.1);
		//moveNormalCenter();
	}
	mesh->computeVertexFromCenter();
	/*
	setNormalAsGraCeneter(func);
	for(i=0; i<iter; i++){	
		
	}*/
	//setNormalAsGrad(func);
	//mesh->computeVertexFromCenter();
	mesh->computeFaceNormal();
}

void Smoother::moveNormalCenter()
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;
	double (*dp)[3] = new double[vertex_N][3];
	int **link_f = mesh->vertex_link_f;
	int **link_v = mesh->vertex_link_v;
	int *degree_f = mesh->degree_f;
	int *degree_v = mesh->degree_v;
	float (*center)[3] = mesh->center;

	for(int i=0; i<vertex_N; i++){
		if(mesh->isFixed[i])
			continue;

		if(mesh->isBound[i] == NON_MANIFOLD){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;

			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.001)
				continue;

			int f1 = link_f[i][0];
			int f2 = link_f[i][degree_f[i]-1];

			float n1[3], n2[3];
			n1[0] = normal_f[f1][0];
			n1[1] = normal_f[f1][1];
			n1[2] = normal_f[f1][2];

			n2[0] = normal_f[f2][0];
			n2[1] = normal_f[f2][1];
			n2[2] = normal_f[f2][2];

			double dot1 = MeshData::DOT(n1, n)/len;
			double dot2 = MeshData::DOT(n2, n)/len;

			n1[0] -= dot1*n[0]/len;
			n1[1] -= dot1*n[1]/len;
			n1[2] -= dot1*n[2]/len;
			double len1 = MeshData::LENGTH(n1);
			if((float)len1 == 0)
				continue;
			n1[0] /= len1;
			n1[1] /= len1;
			n1[2] /= len1;

			n2[0] -= dot2*n[0]/len;
			n2[1] -= dot2*n[1]/len;
			n2[2] -= dot2*n[2]/len;
			double len2 = MeshData::LENGTH(n2);
			if((float)len2 == 0)
				continue;
			n2[0] /= len2;
			n2[1] /= len2;
			n2[2] /= len2;

			dot1 = 0.5*MeshData::DOT(v1, n1);
			dot2 = 0.5*MeshData::DOT(v2, n2);

			len1 = MeshData::LENGTH(v1);
			len2 = MeshData::LENGTH(v2);
			len = len1 + len2;

			if((float)len != 0){
				dp[i][0] = (len1*dot1*n1[0] + len2*dot2*n2[0])/len;
				dp[i][1] = (len1*dot1*n1[1] + len2*dot2*n2[1])/len;
				dp[i][2] = (len1*dot1*n1[2] + len2*dot2*n2[2])/len;
			}
			continue;
		}

		int *nei, *nei_v, nei_N;
		mesh->getConsistent1Ring(i, nei_v, nei, nei_N);
		if(nei_N == 0){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		double total_w = 0;

		double pro[3];
		pro[0] = pro[1] = pro[2] = 0;

		BOOL flip_flag = false;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = face[f][0];
			int i1 = face[f][1];
			int i2 = face[f][2];

			double o[3];
			o[0] = center[f][0];
			o[1] = center[f][1];
			o[2] = center[f][2];

			double d[3];
			MeshData::VEC(d, o, vertex[i]);
			double inner = MeshData::DOT(normal_f[f], d);

			/*
			float v1[3], v2[3];
			if(i0 == i){
				MeshData::VEC(v1, vertex[i0], vertex[i1]);
				MeshData::VEC(v2, vertex[i0], vertex[i2]);
			}
			else if(i1 == i){
				MeshData::VEC(v1, vertex[i1], vertex[i0]);
				MeshData::VEC(v2, vertex[i1], vertex[i2]);
			}
			else{
				MeshData::VEC(v1, vertex[i2], vertex[i0]);
				MeshData::VEC(v2, vertex[i2], vertex[i1]);
			}
			double len1 = MeshData::LENGTH(v1);
			double len2 = MeshData::LENGTH(v2);

			float w;
			if((float)len1 == 0 || (float)len2 == 0){
				w = 0;
			}
			else{
				double dot = MeshData::DOT(v1, v2)/(len1*len2);
				if(dot > 1.0)
					dot = 1;
				else if(dot < -1.0)
					dot = -1;
				w = acos(dot);
			}*/
			double w = MeshData::AREA(vertex[i0], vertex[i1], vertex[i2]);

			total_w += w;

			pro[0] -= w*inner*normal_f[f][0];
			pro[1] -= w*inner*normal_f[f][1];
			pro[2] -= w*inner*normal_f[f][2];

			//float n[3];
			//float v1[3];
			//float v2[3];
			//MeshData::VEC(v1, vertex[i0], vertex[i1]);
			//MeshData::VEC(v2, vertex[i0], vertex[i2]);
			//MeshData::CROSS(n, v1, v2);
			//if(MeshData::DOT(normal_f[f], n) > 0.7){
				//flip_flag = true;
				//break;
				//TRACE("FLIP\n");
			//}
		}
		if(flip_flag){
			//mesh->laplacian(i, dp[i]);
			/*
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			for(int j=0; j<nei_N; j++){
				dp[i][0] += vertex[nei_v[j]][0] - vertex[i][0];
				dp[i][1] += vertex[nei_v[j]][1] - vertex[i][0];
				dp[i][2] += vertex[nei_v[j]][2] - vertex[i][0];
			}
			dp[i][0] /= (float)nei_N;
			dp[i][1] /= (float)nei_N;
			dp[i][2] /= (float)nei_N;*/
		}
		else if(total_w != 0){
			dp[i][0] = pro[0]/total_w;
			dp[i][1] = pro[1]/total_w;
			dp[i][2] = pro[2]/total_w;
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}

	//mesh->computeFaceNormal();
	//mesh->computeNormal();
	for(i=0; i<vertex_N; i++){
		if(mesh->isFixed[i])
			continue;

		//if(MeshData::DOT(dp[i], mesh->normal[i]) > 0 && !isSolidBound[i]){
			vertex[i][0] += dp[i][0];
			vertex[i][1] += dp[i][1];
			vertex[i][2] += dp[i][2];
		//}
	}
	delete[] dp;
}

void Smoother::setNormalAsGraCeneter(vtkImplicitFunction *func)
{
	int face_N = mesh->face_N;
	int (*face)[3] = mesh->face;
	float (*center)[3] = mesh->center;
	float (*normal)[3] = mesh->normal_f;
	if(normal == NULL)
		normal = mesh->normal_f = new float[face_N][3];

	for(int i=0; i<face_N; i++){
		float g[3];
		func->EvaluateGradient(center[i], g);
		double len = MeshData::LENGTH(g);
		if(len != 0){	
			normal[i][0] = (float)g[0]/len;
			normal[i][1] = (float)g[1]/len;
			normal[i][2] = (float)g[2]/len;
		}
		/*
		else{
			normal[i][0] = normal[i][1] = normal[i][2] = 0;
		}*/
	}
}

void Smoother::moveNormalMesh(vtkImplicitFunction* func)
{
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;
	double (*dp)[3] = new double[table_N][3];
	int **link_f = mesh->vertex_link_f;
	int **link_v = mesh->vertex_link_v;
	int *degree_f = mesh->degree_f;
	int *degree_v = mesh->degree_v;

	for(int m=0; m<table_N; m++){
		int i = v_table[m];

		if(mesh->isBound[i] == NON_MANIFOLD){
			dp[m][0] = dp[m][1] = dp[m][2] = 0;
			continue;
		}

		if(mesh->isBound[i]){
			dp[m][0] = dp[m][1] = dp[m][2] = 0;

			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.001)
				continue;

			int f1 = link_f[i][0];
			int f2 = link_f[i][degree_f[i]-1];

			float n1[3], n2[3];
			n1[0] = normal_f[f1][0];
			n1[1] = normal_f[f1][1];
			n1[2] = normal_f[f1][2];

			n2[0] = normal_f[f2][0];
			n2[1] = normal_f[f2][1];
			n2[2] = normal_f[f2][2];

			double dot1 = MeshData::DOT(n1, n)/len;
			double dot2 = MeshData::DOT(n2, n)/len;

			n1[0] -= dot1*n[0]/len;
			n1[1] -= dot1*n[1]/len;
			n1[2] -= dot1*n[2]/len;
			double len1 = MeshData::LENGTH(n1);
			if((float)len1 == 0)
				continue;
			n1[0] /= len1;
			n1[1] /= len1;
			n1[2] /= len1;

			n2[0] -= dot2*n[0]/len;
			n2[1] -= dot2*n[1]/len;
			n2[2] -= dot2*n[2]/len;
			double len2 = MeshData::LENGTH(n2);
			if((float)len2 == 0)
				continue;
			n2[0] /= len2;
			n2[1] /= len2;
			n2[2] /= len2;

			dot1 = 0.5*MeshData::DOT(v1, n1);
			dot2 = 0.5*MeshData::DOT(v2, n2);

			len1 = MeshData::LENGTH(v1);
			len2 = MeshData::LENGTH(v2);
			len = len1 + len2;

			if((float)len != 0){
				dp[m][0] = (len1*dot1*n1[0] + len2*dot2*n2[0])/len;
				dp[m][1] = (len1*dot1*n1[1] + len2*dot2*n2[1])/len;
				dp[m][2] = (len1*dot1*n1[2] + len2*dot2*n2[2])/len;
			}
			continue;
		}

		int *nei, *nei_v, nei_N;
		mesh->getConsistent1Ring(i, nei_v, nei, nei_N);
		if(nei_N == 0){
			dp[m][0] = dp[m][1] = dp[m][2] = 0;
			continue;
		}

		double total_w = 0;

		double pro[3];
		pro[0] = pro[1] = pro[2] = 0;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = face[f][0];
			int i1 = face[f][1];
			int i2 = face[f][2];

			double o[3];
			o[0] = (vertex[i0][0] + vertex[i1][0] + vertex[i2][0])/3.0;
			o[1] = (vertex[i0][1] + vertex[i1][1] + vertex[i2][1])/3.0;
			o[2] = (vertex[i0][2] + vertex[i1][2] + vertex[i2][2])/3.0;

			double d[3];
			MeshData::VEC(d, o, vertex[i]);
			double inner = MeshData::DOT(normal_f[f], d);

			double w = MeshData::AREA(vertex[i0], vertex[i1], vertex[i2]);

			total_w += w;

			pro[0] -= w*inner*normal_f[f][0];
			pro[1] -= w*inner*normal_f[f][1];
			pro[2] -= w*inner*normal_f[f][2];
		}
		if(total_w != 0){
			dp[m][0] = pro[0]/total_w;
			dp[m][1] = pro[1]/total_w;
			dp[m][2] = pro[2]/total_w;
		}
		else
			dp[m][0] = dp[m][1] = dp[m][2] = 0;
	}

	//mesh->computeFaceNormal();
	//mesh->computeNormal();

	for(m=0; m<table_N; m++){
		int i = v_table[m];

		if(isSolidBound[i])
			continue;

		double dot = func->EvaluateFunction(vertex[i])*MeshData::DOT(dp[m], mesh->normal[i]);

		if(isEngrave){
			if(dot > 0){
				vertex[i][0] += dp[m][0];
				vertex[i][1] += dp[m][1];
				vertex[i][2] += dp[m][2];
			}
		}
		else if(dot < 0){
			vertex[i][0] += dp[m][0];
			vertex[i][1] += dp[m][1];
			vertex[i][2] += dp[m][2];
		}
	}
	delete[] dp;
}

void Smoother::keepOnSurfaceMesh(vtkImplicitFunction *func)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;
	int **link_v = mesh->vertex_link_v;
	int *degree_v = mesh->degree_v;
	
	float max_V = 0;
	for(int m=0; m<table_N; m++){
		int i = v_table[m];

		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float f = func->EvaluateFunction(vertex[i]);
		float len = MeshData::LENGTH(g);
		float V = fabs(f*len);
	
		if(V > max_V)
			max_V = V;
	}
	if(max_V == 0)
		return;

	float dt = 0.005f/max_V;

	for(m=0; m<table_N; m++){
		int i = v_table[m];

		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float f = func->EvaluateFunction(vertex[i]);

		if(mesh->isBound[i]){
			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.0001)
				continue;
			
			double dot = MeshData::DOT(n, g)/len;
			
			vertex[i][0] -= f*(g[0] - dot*n[0]/len)*dt;
			vertex[i][1] -= f*(g[1] - dot*n[1]/len)*dt;
			vertex[i][2] -= f*(g[2] - dot*n[2]/len)*dt;
			
			continue;
		}

		if(isSolidBound[i]){
			float v[3];
			v[0] = -f*g[0]*dt;
			v[1] = -f*g[1]*dt;
			v[2] = -f*g[2]*dt;
			float inner = MeshData::DOT(originalNormal[i], v);
			vertex[i][0] += v[0] - inner*originalNormal[i][0];
			vertex[i][1] += v[1] - inner*originalNormal[i][1];
			vertex[i][2] += v[2] - inner*originalNormal[i][2];
			continue;
		}

		double dot = f*MeshData::DOT(normal[i], g);
		if(isEngrave){
			if(dot < 0){
				vertex[i][0] -= f*g[0]*dt;
				vertex[i][1] -= f*g[1]*dt;
				vertex[i][2] -= f*g[2]*dt;
			}
		}
		else{
			if(dot > 0){
				vertex[i][0] -= f*g[0]*dt;
				vertex[i][1] -= f*g[1]*dt;
				vertex[i][2] -= f*g[2]*dt;
			}
		}
	}
}

void Smoother::tangentLapMesh(float dt)
{
	int vertex_N = mesh->vertex_N;
	double (*dp)[3] = new double[table_N][3];
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;
	int **link_v = mesh->vertex_link_v;
	int *degree_v = mesh->degree_v;

	for(int m=0; m<table_N; m++){
		int i = v_table[m];

		if(isSolidBound[i])
			continue;

		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		if(mesh->isBound[i]){
			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);
			double len1 = MeshData::LENGTH(v1);
			double len2 = MeshData::LENGTH(v2);
			double n[3];
			n[0] = len2*v1[0] + len1*v2[0];
			n[1] = len2*v1[1] + len1*v2[1];
			n[2] = len2*v1[2] + len1*v2[2];
			double len = MeshData::LENGTH(n);

			if((float)len < 0.001){
				dp[m][0] = dp[m][1] = dp[m][2] = 0;
				continue;
			}

			double v[3];
			v[0] = 0.5*(v1[0] + v2[0]);
			v[1] = 0.5*(v1[1] + v2[1]);
			v[2] = 0.5*(v1[2] + v2[2]);

			double dot = MeshData::DOT(v, n)/len;

			dp[m][0] = v[0] - dot*n[0]/len;
			dp[m][1] = v[1] - dot*n[1]/len;
			dp[m][2] = v[2] - dot*n[2]/len;
			
			continue;
		}
		double L[3];
		mesh->laplacian(i, L);
		float inner = MeshData::DOT(normal[i], L);
		dp[m][0] = L[0] - inner*normal[i][0];
		dp[m][1] = L[1] - inner*normal[i][1];
		dp[m][2] = L[2] - inner*normal[i][2];
	}
		
	for(m=0; m<table_N; m++){
		int i = v_table[m];

		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		if(isSolidBound[i])
			continue;

		vertex[i][0] += dt*dp[m][0];
		vertex[i][1] += dt*dp[m][1];
		vertex[i][2] += dt*dp[m][2];
	}

	delete[] dp;
}

void Smoother::moveImplicit(vtkImplicitFunction *func, int iter, BOOL isEngrave)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	this->isEngrave = isEngrave;
	int **link = mesh->vertex_link_v;
	int *degree = mesh->degree_v;
	
	table_N = 0;
	for(int i=0; i<vertex_N; i++){
		if(func->EvaluateFunction(vertex[i]) >= 0)
			table_N++;
	}
	v_table = new int[table_N];
	BOOL* isInside = new BOOL[vertex_N];
	int count = 0;
	for(i=0; i<vertex_N; i++){
		if(func->EvaluateFunction(vertex[i]) >= 0){
			v_table[count++] = i;
			isInside[i] = true;
		}
		else
			isInside[i] = false;
	}

	isSolidBound = new BOOL[vertex_N];
	for(int m=0; m<table_N; m++){
		i = v_table[m];
		int deg = degree[i];
		int *l = link[i];
		BOOL flag = true;
		for(int j=0; j<deg; j++)
			flag &= isInside[l[j]];
		if(!flag)
			isSolidBound[i] = true;
		else
			isSolidBound[i] = false;
	}

	delete[] isInside;

	mesh->computeNormal();
	originalNormal = new float[vertex_N][3];
	for(i=0; i<vertex_N; i++)
		if(isSolidBound[i]){
			originalNormal[i][0] = mesh->normal[i][0];
			originalNormal[i][1] = mesh->normal[i][1];
			originalNormal[i][2] = mesh->normal[i][2];
		}

	

	for(i=0; i<iter; i++){
		//LapMesh(0.1);
		//continue;
		//LapMesh(-0.6);
		
		//if(i < iter/2){
			//mesh->computeNormal();
			//offset(func);
			//mesh->computeNormal();
			//tangentLapMesh(0.1);
		//}
		setNormalAsGrad(func);
		mesh->computeNormal();
		moveNormalMesh(func);
		//if(i < iter/2){
			//mesh->computeNormal();
			//tangentLapMesh(0.1);
		//}
		mesh->computeNormal();
		keepOnSurfaceMesh(func);
	
		/*
		if(uniform == 0 && i < iter/2){
			mesh->computeFaceNormal();
			mesh->computeNormal();
			//setVertexNormalAsGrad(func);
			//tangentLapW(0.1, T);
			tangentLap(0.1, from);
		}
		else if(uniform == 1){
			mesh->computeFaceNormal();
			mesh->computeNormal();
			//tangentLap(0.1);
			//setVertexNormalAsGrad(func);
			tangentLap(1.0f - (float)i/iter, from); //, T);
			//tangentLap(0.1, from);
		}
			
			
		if(keep == 0){
			//mesh->computeFaceNormal();
			//mesh->computeNormal();
			keepOnSurface(func, from);
		}*/
	}

	delete[] isSolidBound;
	delete[] originalNormal;
	delete[] v_table;

	mesh->computeFaceNormal();
}

void Smoother::LapMesh(float dt)
{
	double (*dp)[3] = new double[table_N][3];
	float (*vertex)[3] = mesh->vertex;

	for(int m=0; m<table_N; m++){
		int i = v_table[m];

		//if(isSolidBound[i])
			//continue;

		mesh->laplacian(i, dp[m]);
	}
	for(m=0; m<table_N; m++){
		int i = v_table[m];

		vertex[i][0] += dt*dp[m][0];
		vertex[i][1] += dt*dp[m][1];
		vertex[i][2] += dt*dp[m][2];
	}
	delete[] dp;
}

void Smoother::offset(vtkImplicitFunction *func)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal;
	int **link_v = mesh->vertex_link_v;
	int *degree_v = mesh->degree_v;
	
	float max_V = 0;
	for(int m=0; m<table_N; m++){
		int i = v_table[m];

		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float f = func->EvaluateFunction(vertex[i]);
		float len = MeshData::LENGTH(g);
		float V = fabs(f*len);
	
		if(V > max_V)
			max_V = V;
	}
	if(max_V == 0)
		return;

	float dt = 0.005f/max_V;

	for(m=0; m<table_N; m++){
		int i = v_table[m];

		if(mesh->isBound[i] == NON_MANIFOLD)
			continue;

		float g[3];
		func->EvaluateGradient(vertex[i], g);
		float f = func->EvaluateFunction(vertex[i]);

		if(mesh->isBound[i]){
			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.0001)
				continue;
			
			double dot = MeshData::DOT(n, g)/len;
			
			vertex[i][0] -= f*(g[0] - dot*n[0]/len)*dt;
			vertex[i][1] -= f*(g[1] - dot*n[1]/len)*dt;
			vertex[i][2] -= f*(g[2] - dot*n[2]/len)*dt;
			
			continue;
		}

		if(isSolidBound[i]){
			continue;
		}

		double dot = MeshData::DOT(normal[i], g);
		if(isEngrave){
			float len = MeshData::LENGTH(g);
			vertex[i][0] += f*len*dt*normal[i][0];
			vertex[i][1] += f*len*dt*normal[i][1];
			vertex[i][2] += f*len*dt*normal[i][2];
		}
		else{
			float len = MeshData::LENGTH(g);
			vertex[i][0] -= f*len*dt*normal[i][0];
			vertex[i][1] -= f*len*dt*normal[i][1];
			vertex[i][2] -= f*len*dt*normal[i][2];
		}
	}
}

void Smoother::keepOnSurfaceT(vtkImplicitFunction *func, float c)
{
	int vertex_N = mesh->vertex_N;
	int face_N = mesh->face_N;
	double (*dp)[3] = new double[vertex_N][3];
	int (*face)[3] = mesh->face;
	float (*vertex)[3] = mesh->vertex;
	double max_len = 0;
	float (*normal)[3] = mesh->normal_f;

	for(int i=0; i<vertex_N; i++)
		dp[i][0] = dp[i][1] = dp[i][2] = 0;

	for(i=0; i<face_N; i++){
		int i1 = face[i][0];
		int i2 = face[i][1];
		int i3 = face[i][2];

		float cen[3];
		mesh->faceCenter(cen, i);

		float v1[3], v2[3];
		MeshData::VEC(v1, vertex[i1], vertex[i2]);
		MeshData::VEC(v2, vertex[i1], vertex[i3]);

		double cv[3];
		MeshData::CROSS(cv, v1, v2);
		double a = MeshData::LENGTH(cv);
	
		float v = func->EvaluateFunction(cen);

		//float dv[3];
		//func->EvaluateGradient(cen, dv);

		float *dv = normal[i];

		dp[i1][0] += a*v*dv[0]/3.0;
		dp[i1][1] += a*v*dv[1]/3.0;
		dp[i1][2] += a*v*dv[2]/3.0;

		dp[i2][0] += a*v*dv[0]/3.0;
		dp[i2][1] += a*v*dv[1]/3.0;
		dp[i2][2] += a*v*dv[2]/3.0;

		dp[i3][0] += a*v*dv[0]/3.0;
		dp[i3][1] += a*v*dv[1]/3.0;
		dp[i3][2] += a*v*dv[2]/3.0;

		double len = a*fabs(v)*MeshData::LENGTH(dv)/3.0;
		if(len > max_len)
			max_len = len;
	}

	for(i=0; i<vertex_N; i++){
		vertex[i][0] += (float)(c*dp[i][0]/max_len);
		vertex[i][1] += (float)(c*dp[i][1]/max_len);
		vertex[i][2] += (float)(c*dp[i][2]/max_len);
	}

	delete[] dp;
}

void Smoother::keepOnSurfaceUT(vtkImplicitFunction *func, float c1, float c2, float T)
{
	int vertex_N = mesh->vertex_N;
	int face_N = mesh->face_N;
	double (*dp)[3] = new double[vertex_N][3];
	float (*df)[3] = new float[face_N][3];
	int (*face)[3] = mesh->face;
	float (*vertex)[3] = mesh->vertex;
	double max_len = 0;
	int **link_f = mesh->vertex_link_f;
	int **link_v = mesh->vertex_link_v;
	int *degree = mesh->degree_f;

	for(int i=0; i<vertex_N; i++)
		dp[i][0] = dp[i][1] = dp[i][2] = 0;

	for(i=0; i<face_N; i++){
		int i1 = face[i][0];
		int i2 = face[i][1];
		int i3 = face[i][2];

		float cen[3];
		mesh->faceCenter(cen, i);

		float v1[3], v2[3];
		MeshData::VEC(v1, vertex[i1], vertex[i2]);
		MeshData::VEC(v2, vertex[i1], vertex[i3]);

		double cv[3];
		MeshData::CROSS(cv, v1, v2);
		double a = MeshData::LENGTH(cv);
	
		float v = func->EvaluateFunction(cen);

		func->EvaluateGradient(cen, df[i]);
		double len = fabs(v)*MeshData::LENGTH(df[i])/3.0;
		if(len > max_len)
			max_len = len;

		df[i][0] *= a;
		df[i][1] *= a;
		df[i][2] *= a;
		double z[3];
		z[0] = df[i][0]*v/3.0;
		z[1] = df[i][1]*v/3.0;
		z[2] = df[i][2]*v/3.0;

		dp[i1][0] += z[0];
		dp[i1][1] += z[1];
		dp[i1][2] += z[2];

		dp[i2][0] += z[0];
		dp[i2][1] += z[1];
		dp[i2][2] += z[2];

		dp[i3][0] += z[0];
		dp[i3][1] += z[1];
		dp[i3][2] += z[2];
	}

	for(i=0; i<vertex_N; i++){
		int deg = degree[i];
		int* f = link_f[i];
		int sharp = 0;
		int e1, e2;
		double Lt[3];
		double n[3] = {0,0,0};
		for(int j=0; j<deg; j++){
			int fi = f[j];
			int fj = f[(j+1)/deg];
			if(MeshData::DOT(df[fi], df[fj])/(MeshData::LENGTH(df[fi])*MeshData::LENGTH(df[fj])) < T){
				sharp++;
				if(sharp == 1)
					e1 = (j+1)%deg;
				else if(sharp == 2)
					e2 = (j+1)%deg;
			}
			n[0] += df[fi][0];
			n[1] += df[fi][1];
			n[2] += df[fi][2];
		}
		double len = MeshData::LENGTH(n);
		if((float)len != 0){
			n[0] /= len;
			n[1] /= len;
			n[2] /= len;
		}
		else
			n[0] = n[1] = n[2] = 0;

		if(sharp == 0){
			double L[3];
			mesh->laplacian(i,L);
			double dot = MeshData::DOT(L, n);
			Lt[0] = L[0] - dot*n[0];
			Lt[1] = L[1] - dot*n[1];
			Lt[2] = L[2] - dot*n[2];
		}
		else if(sharp == 2){
			double L[3];
			float* p1 = vertex[link_v[i][e1]];
			float* p2 = vertex[link_v[i][e2]];
			L[0] = 0.5*(p1[0] + p2[0]) - vertex[i][0];
			L[1] = 0.5*(p1[1] + p2[1]) - vertex[i][1];
			L[2] = 0.5*(p1[2] + p2[2]) - vertex[i][2];
			double dot = MeshData::DOT(L, n);
			Lt[0] = L[0] - dot*n[0];
			Lt[1] = L[1] - dot*n[1];
			Lt[2] = L[2] - dot*n[2];
		}
		else
			Lt[0] = Lt[1] = Lt[2] = 0;

		dp[i][0] = -c1*dp[i][0]/max_len + c2*Lt[0];
		dp[i][1] = -c1*dp[i][1]/max_len + c2*Lt[1];
		dp[i][2] = -c1*dp[i][2]/max_len + c2*Lt[2];
	}

	for(i=0; i<vertex_N; i++){
		vertex[i][0] += (float)dp[i][0];
		vertex[i][1] += (float)dp[i][1];
		vertex[i][2] += (float)dp[i][2];
	}

	delete[] dp;
	delete[] df;
}

void Smoother::keepOnSurfaceUTN(vtkImplicitFunction *func, float c1, float c2, float c3, float T)
{
	int vertex_N = mesh->vertex_N;
	int face_N = mesh->face_N;
	double (*dp)[3] = new double[vertex_N][3];
	float (*df)[3] = new float[face_N][3];
	int (*face)[3] = mesh->face;
	float (*vertex)[3] = mesh->vertex;
	double max_len = 0;
	int **link_f = mesh->vertex_link_f;
	int **link_v = mesh->vertex_link_v;
	int *degree = mesh->degree_f;
	float (*normal)[3] = mesh->normal_f;

	for(int i=0; i<vertex_N; i++)
		dp[i][0] = dp[i][1] = dp[i][2] = 0;

	for(i=0; i<face_N; i++){
		int i1 = face[i][0];
		int i2 = face[i][1];
		int i3 = face[i][2];

		float cen[3];
		mesh->faceCenter(cen, i);

		float v1[3], v2[3];
		MeshData::VEC(v1, vertex[i1], vertex[i2]);
		MeshData::VEC(v2, vertex[i1], vertex[i3]);

		double cv[3];
		MeshData::CROSS(cv, v1, v2);
		double a = MeshData::LENGTH(cv);
		normal[i][0] = (float)(cv[0]/a);
		normal[i][1] = (float)(cv[1]/a);
		normal[i][2] = (float)(cv[2]/a);
	
		float v = func->EvaluateFunction(cen);

		func->EvaluateGradient(cen, df[i]);
		double len = fabs(v)*MeshData::LENGTH(df[i])/3.0;
		if(len > max_len)
			max_len = len;

		df[i][0] *= a;
		df[i][1] *= a;
		df[i][2] *= a;
		double z[3];
		z[0] = df[i][0]*v/3.0;
		z[1] = df[i][1]*v/3.0;
		z[2] = df[i][2]*v/3.0;

		dp[i1][0] += z[0];
		dp[i1][1] += z[1];
		dp[i1][2] += z[2];

		dp[i2][0] += z[0];
		dp[i2][1] += z[1];
		dp[i2][2] += z[2];

		dp[i3][0] += z[0];
		dp[i3][1] += z[1];
		dp[i3][2] += z[2];
	}

	for(i=0; i<vertex_N; i++){
		int deg = degree[i];
		int* f = link_f[i];
		int sharp = 0;
		int e1, e2;
		double Lt[3];
		double n[3] = {0,0,0};
		for(int j=0; j<deg; j++){
			int fi = f[j];
			int fj = f[(j+1)/deg];
			if(MeshData::DOT(df[fi], df[fj])/(MeshData::LENGTH(df[fi])*MeshData::LENGTH(df[fj])) < T){
				sharp++;
				if(sharp == 1)
					e1 = (j+1)%deg;
				else if(sharp == 2)
					e2 = (j+1)%deg;
			}
			n[0] += df[fi][0];
			n[1] += df[fi][1];
			n[2] += df[fi][2];
		}
		double len = MeshData::LENGTH(n);
		if((float)len != 0){
			n[0] /= len;
			n[1] /= len;
			n[2] /= len;
		}
		else
			n[0] = n[1] = n[2] = 0;

		if(sharp == 0){
			double L[3];
			mesh->laplacian(i,L);
			double dot = MeshData::DOT(L, n);
			Lt[0] = L[0] - dot*n[0];
			Lt[1] = L[1] - dot*n[1];
			Lt[2] = L[2] - dot*n[2];
		}
		else if(sharp == 2){
			double L[3];
			float* p1 = vertex[link_v[i][e1]];
			float* p2 = vertex[link_v[i][e2]];
			L[0] = 0.5*(p1[0] + p2[0]) - vertex[i][0];
			L[1] = 0.5*(p1[1] + p2[1]) - vertex[i][1];
			L[2] = 0.5*(p1[2] + p2[2]) - vertex[i][2];
			double dot = MeshData::DOT(L, n);
			Lt[0] = L[0] - dot*n[0];
			Lt[1] = L[1] - dot*n[1];
			Lt[2] = L[2] - dot*n[2];
		}
		else
			Lt[0] = Lt[1] = Lt[2] = 0;

		dp[i][0] = -c1*dp[i][0]/max_len + c2*Lt[0];
		dp[i][1] = -c1*dp[i][1]/max_len + c2*Lt[1];
		dp[i][2] = -c1*dp[i][2]/max_len + c2*Lt[2];
	}

	for(i=0; i<face_N; i++){
		double v[3];
		double len = MeshData::LENGTH(df[i]);
		v[0] = df[i][0]/len - normal[i][0];
		v[1] = df[i][1]/len - normal[i][1];
		v[2] = df[i][2]/len - normal[i][2];

		int i1 = face[i][0];
		int i2 = face[i][1];
		int i3 = face[i][2];

		//vertex i1
		float *p1, *p2;
		p1 = vertex[i2];
		p2 = vertex[i3];

		float cx[3], cy[3], cz[3];
		cx[0] = 0;
		cx[1] = p1[2] - p2[2];
		cx[2] = p2[1] - p1[1];
			
		cy[0] = p2[2] - p1[2];
		cy[1] = 0;
		cy[2] = p1[0] - p2[0];

		cz[0] = p1[1] - p2[1];
		cz[1] = p2[0] - p1[0];
		cz[2] = 0;

		dp[i1][0] += c3*MeshData::DOT(v, cx);
		dp[i1][1] += c3*MeshData::DOT(v, cy);  
		dp[i1][2] += c3*MeshData::DOT(v, cz); 

		//vertex i2
		p1 = vertex[i3];
		p2 = vertex[i1];

		cx[0] = 0;
		cx[1] = p1[2] - p2[2];
		cx[2] = p2[1] - p1[1];
			
		cy[0] = p2[2] - p1[2];
		cy[1] = 0;
		cy[2] = p1[0] - p2[0];

		cz[0] = p1[1] - p2[1];
		cz[1] = p2[0] - p1[0];
		cz[2] = 0;

		dp[i2][0] += c3*MeshData::DOT(v, cx);
		dp[i2][1] += c3*MeshData::DOT(v, cy);  
		dp[i2][2] += c3*MeshData::DOT(v, cz); 

		//vertexi3
		p1 = vertex[i1];
		p2 = vertex[i2];

		cx[0] = 0;
		cx[1] = p1[2] - p2[2];
		cx[2] = p2[1] - p1[1];
			
		cy[0] = p2[2] - p1[2];
		cy[1] = 0;
		cy[2] = p1[0] - p2[0];

		cz[0] = p1[1] - p2[1];
		cz[1] = p2[0] - p1[0];
		cz[2] = 0;

		dp[i3][0] += c3*MeshData::DOT(v, cx);
		dp[i3][1] += c3*MeshData::DOT(v, cy);  
		dp[i3][2] += c3*MeshData::DOT(v, cz); 
	}

	for(i=0; i<vertex_N; i++){
		vertex[i][0] += (float)dp[i][0];
		vertex[i][1] += (float)dp[i][1];
		vertex[i][2] += (float)dp[i][2];
	}

	delete[] dp;
	delete[] df;
}

void Smoother::correctImplicitSurfacePowell(OurImplicitFunction *func)
{
	/*
	computeCenterBisection(0.1, func, 40);
	setVertexFromCenter();
	optimizeSquentily(func, 0.01);
	*/

	int i;

	int face_N = mesh->face_N;
	center = new float[face_N][3];
	valid = new BOOL[face_N];
	m = new float[face_N][3];

	/*
	for(int i=0; i<face_N; i++)
		mesh->faceCenter(center[i], i);

	computeCenterBisection(0.1, func, 20);
	fillValid();
	for(i=0; i<10; i++){	
		setCenterNormal(func);
		uniformizeCenter(0.5);
	}
	computeCenterBisection(0.1, func, 20);
	fillValid();
	setCenterNormal(func);*/

	
	for(i=0; i<5; i++){
		for(int j=0; j<face_N; j++)
			mesh->faceCenter(center[j], j);
		computeCenterBisection(0.1, func, 20);
		//setVertexFromCenter();
		setCenterNormal(func);
		setVertexFromCenterW(5.0f);
	}

	/*
	for(int j=0; j<face_N; j++)
		mesh->faceCenter(center[j], j);
	computeCenterBisection(0.1, func, 20);
	//fillValid();
	setVertexFromCenter();
	//setVertexFromCenterW(5.0f);
	resolveFlip();
	setCenterNormal(func);
*/
	//setVertexFromCenter();
	//setCenterNormal(func);

	//setCenterNormal(func);

	//for(i=0; i<50; i++)
		//moveNormalCenter2();

	double (*Q)[10] = new double[mesh->vertex_N][10];
	computeQuadric(Q);
	//averagingQ(20, Q);
	minimizeQ(50, Q);
	delete[] Q;

	delete[] center;
	delete[] valid;
	delete[] m;

	/*
	float err = 0;
	for(int i=0; i<mesh->face_N; i++){
		float f = func->EvaluateFunction(center[i]);
		if(f*f > 1)
			int a = 0;
		err += f*f;
	}
	*/
	/*
	setCenterNormal(func);
	setOpt();
	*/
	/*
	setCenterNormal(func);
	for(int i=0; i<20; i++)
		moveNormalCenter2();
	*/
		
	/*
	LocalH1Error e;
	e.f = func;
	e.mesh = mesh;

	EnergyMinimizer mini;
	mini.energy = &e;

	for(int i=0; i< mesh->vertex_N; i++){
		//int i = 280;
		e.index = i;
		int iter;
		float fret, fret1;
		fret1 = e.func(mesh->vertex[i]);
		if(fret1 > 0.0001)
			mini.frprmn(mesh->vertex[i], 3, 0.00001, &iter, &fret);
		int a = 0;
	}
	*/
}

void Smoother::computeCenterBisection(float interval, vtkImplicitFunction* func, int MAX)
{
	int face_N = mesh->face_N;
	for(int i=0; i<face_N; i++){
		float c[3];
		double area = mesh->triangleEdgeAverage(i); //mesh->faceArea(i);
		valid[i] = searchNearPoint(func, center[i], c, 0.5*area, MAX);
		//valid[i] = searchNearPoint1D(func, center[i], c, m[i], 0.5*area, MAX);
		/***********/
		float c1[3];
		c1[0] = 2*center[i][0] - c[0];
		c1[1] = 2*center[i][1] - c[1];
		c1[2] = 2*center[i][2] - c[2];
		if((func->EvaluateFunction(center[i]))*(func->EvaluateFunction(c1)) < 0){
			float d[3];
			d[0] = center[i][0] - c[0];
			d[1] = center[i][1] - c[1];
			d[2] = center[i][2] - c[2];
			double l = MeshData::LENGTH(d);
			if(d != 0){
				d[0] /= -l;
				d[1] /= -l;
				d[2] /= -l;
				valid[i] = searchNearPoint1D(func, center[i], c1, d, 0.5*area, MAX);
				if(MeshData::DIST(center[i], c1) < MeshData::DIST(center[i], c)){
					c[0] = c1[0];
					c[1] = c1[1];
					c[2] = c1[2];
				}
			}
		}
		/***********/
		if(valid[i]){
			if(MeshData::DIST(c, center[i]) > area)
				valid[i] = false;
			else{
				center[i][0] = c[0];
				center[i][1] = c[1];
				center[i][2] = c[2];
			}
		}
	}
	/*
	for(int i=0; i<face_N; i++){
		float c[3];
		mesh->faceCenter(c, i);
		float p1[3], p2[3], f1, f2;
		p1[0] = p2[0] = c[0];
		p1[1] = p2[1] = c[1];
		p1[2] = p2[2] = c[2];
		f1 = func->EvaluateFunction(p1);
		if(fabs(f1) < 0.00001){
			center[i][0] = c[0];
			center[i][1] = c[1];
			center[i][2] = c[2];
			continue;
		}

		float inter = interval;
		if(f1 > 0)
			inter = -inter;
		for(int j=0; j<MAX; j++){
			float d[3];
			func->EvaluateGradient(c, d);
			double len = MeshData::LENGTH(d);
			if(len != 0){
				d[0] /= len;
				d[1] /= len;
				d[2] /= len;
			}
			p2[0] += inter*d[0];
			p2[1] += inter*d[1];
			p2[2] += inter*d[2];
			f2 = func->EvaluateFunction(p2);
			if(f1 * f2 < 0)
				break;
		}
		if(j == MAX){
			center[i][0] = c[0];
			center[i][1] = c[1];
			center[i][2] = c[2];
			continue;
		}

		float p3[3];
		for(j=0; j<MAX; j++){
			p3[0] = 0.5f*(p1[0] + p2[0]);
			p3[1] = 0.5f*(p1[1] + p2[1]);
			p3[2] = 0.5f*(p1[2] + p2[2]);
			float f3 = func->EvaluateFunction(p3);
			if(f3 == 0){
				center[i][0] = p3[0];
				center[i][1] = p3[1];
				center[i][2] = p3[2];
				break;
			}
			else if(f1*f3 >= 0){
				p1[0] = p3[0];
				p1[1] = p3[1];
				p1[2] = p3[2];
				f1 = f3;
			}
			else{
				p2[0] = p3[0];
				p2[1] = p3[1];
				p2[2] = p3[2];
				f2 = f3;
			}
		}
		if(j == MAX){
			center[i][0] = p3[0];
			center[i][1] = p3[1];
			center[i][2] = p3[2];
		}
	}*/
}

void Smoother::setVertexFromCenter()
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int *degree = mesh->degree_f;
	int **link = mesh->vertex_link_f;

	for(int i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		float p[3];
		p[0] = p[1] = p[2] = 0;
		int valid_N = 0;
		for(int j=0; j<deg; j++){
			if(!valid[l[j]])
				continue;
			valid_N++;
			p[0] += center[l[j]][0];
			p[1] += center[l[j]][1];
			p[2] += center[l[j]][2];
		}
		if(valid_N != 0){
			vertex[i][0] = p[0]/valid_N;
			vertex[i][1] = p[1]/valid_N;
			vertex[i][2] = p[2]/valid_N;
		}
	}
}

void Smoother::setCenterNormal(vtkImplicitFunction* func)
{
	int face_N = mesh->face_N;

	for(int i=0; i<face_N; i++){
		func->EvaluateGradient(center[i], m[i]);
		double len = MeshData::LENGTH(m[i]);
		if(len != 0){
			m[i][0] /= len;
			m[i][1] /= len;
			m[i][2] /= len;
		}
	}
}

void Smoother::moveNormalCenter2()
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	double (*dp)[3] = new double[vertex_N][3];
	int **link_f = mesh->vertex_link_f;
	int *degree_f = mesh->degree_f;

	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i] != INNER){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		int *nei = link_f[i];
		int nei_N = degree_f[i];

		if(nei_N == 0){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		double total_w = 0;

		double pro[3];
		pro[0] = pro[1] = pro[2] = 0;

		double uni[3];
		uni[0] = uni[1] = uni[2] = 0;

		BOOL flip_flag = false;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			if(!valid[f])
				continue;

			int i0 = face[f][0];
			int i1 = face[f][1];
			int i2 = face[f][2];

			double o[3];
			o[0] = center[f][0];
			o[1] = center[f][1];
			o[2] = center[f][2];

			double d[3];
			MeshData::VEC(d, o, vertex[i]);
			double inner = MeshData::DOT(m[f], d);

			double w = MeshData::AREA(vertex[i0], vertex[i1], vertex[i2]);

			total_w += w;

			pro[0] -= w*inner*m[f][0];
			pro[1] -= w*inner*m[f][1];
			pro[2] -= w*inner*m[f][2];

			float c[3];
			mesh->faceCenter(c, f);
			uni[0] -= 0.00*w*(c[0] - center[f][0]);
			uni[1] -= 0.00*w*(c[1] - center[f][1]);
			uni[2] -= 0.00*w*(c[2] - center[f][2]);
		}
		
		if(total_w != 0){
			dp[i][0] = (pro[0] + uni[0])/total_w;
			dp[i][1] = (pro[1] + uni[1])/total_w;
			dp[i][2] = (pro[2] + uni[2])/total_w;
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}

	for(i=0; i<vertex_N; i++){
		if(mesh->isFixed[i])
			continue;

		vertex[i][0] += dp[i][0];
		vertex[i][1] += dp[i][1];
		vertex[i][2] += dp[i][2];
	}
	delete[] dp;
}

void Smoother::setOpt()
{
	int face_N = mesh->face_N;
	int vertex_N = mesh->vertex_N;
	int *degree_f = mesh->degree_f;
	int **f_link = mesh->vertex_link_f;
	float (*vertex)[3] = mesh->vertex;
	BOOL *fail = new BOOL[vertex_N];
	float (*optP)[3] = new float[vertex_N][3];
	int (*face)[3] = mesh->face;

	//quadlic computation
	double (*Qf)[10] = new double[face_N][10];
	for(int i=0; i<face_N; i++){
		double mi[3];
		mi[0] = m[i][0];
		mi[1] = m[i][1];
		mi[2] = m[i][2];
		double w = mesh->faceArea(i);
		double d = -MeshData::DOT(mi, center[i]);
		MATRIX(Qf[i], mi, d);
		MAT_TIMES(Qf[i], w);
	}

	for(i=0; i<vertex_N; i++){
		double Q[10];
		int fn = degree_f[i];
		int* link = f_link[i];
		MAT_INIT(Q);
		for(int j=0; j<fn; j++)
			MAT_SUM(Q, Qf[link[j]]);

		double inv[10], opt[3];
		double b[3];
		b[0] = -Q[6];
		b[1] = -Q[7];
		b[2] = -Q[8];

		if(INVERSE(inv, Q))
			MAT_BY_VEC(opt, inv, b);
		else{
			Q[0] += 0.01;
			Q[3] += 0.01;
			Q[5] += 0.01;
			if(INVERSE(inv, Q)){
				b[0] += 0.01*vertex[i][0];
				b[1] += 0.01*vertex[i][1];
				b[2] += 0.01*vertex[i][2];
				MAT_BY_VEC(opt, inv, b);
			}
			else{
				fail[i] = true;
				continue;
			}
		}

		//flip check by "zone" technique
		BOOL flip = false;
		for(j=0; j<fn; j++){
			int f = link[j];

			int i1, i2;
			if(face[f][0] == i){
				i1 = face[f][1];
				i2 = face[f][2];
			}
			else if(face[f][1] == i){
				i1 = face[f][2];
				i2 = face[f][0];
			}
			else{
				i1 = face[f][0];
				i2 = face[f][1];
			}

			float v[3];
			v[0] = vertex[i1][0] - vertex[i2][0];
			v[1] = vertex[i1][1] - vertex[i2][1];
			v[2] = vertex[i1][2] - vertex[i2][2];

			double n[3];
			MeshData::CROSS(n, v, m[f]);
		
			float e[3];
			e[0] = 0.5*(vertex[i1][0] + vertex[i2][0]);
			e[1] = 0.5*(vertex[i1][1] + vertex[i2][1]);
			e[2] = 0.5*(vertex[i1][2] + vertex[i2][2]);

			MeshData::VEC(v, vertex[i], e);
			double side1 = MeshData::DOT(n, v);

			float opt_f[3];
			opt_f[0] = opt[0];
			opt_f[1] = opt[1];
			opt_f[2] = opt[2];
			MeshData::VEC(v, opt_f, e);
			double side2 = MeshData::DOT(n, v);

			if(side1 * side2 <= 0){
				flip = true;
				break;
			}
		}
		if(flip){
			fail[i] = true;
			continue;
		}
		
		optP[i][0] = opt[0];
		optP[i][1] = opt[1];
		optP[i][2] = opt[2];
		fail[i] = false;
	}
	delete[] Qf;

	for(i=0; i<vertex_N; i++){
		if(fail[i]){
			double L[3];
			mesh->laplacian(i, L);
			vertex[i][0] += L[0];
			vertex[i][1] += L[1];
			vertex[i][2] += L[2];
		}
		else{
			vertex[i][0] = optP[i][0];
			vertex[i][1] = optP[i][1];
			vertex[i][2] = optP[i][2];
		}
	}
	delete[] fail;
	delete[] optP;
}

void Smoother::computeVertexBisection(float interval, vtkImplicitFunction *func, int MAX)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	for(int i=0; i<vertex_N; i++){
		float c[3];
		c[0] = vertex[i][0];
		c[1] = vertex[i][1];
		c[2] = vertex[i][2];
		float p1[3], p2[3], f1, f2;
		p1[0] = p2[0] = c[0];
		p1[1] = p2[1] = c[1];
		p1[2] = p2[2] = c[2];
		f1 = func->EvaluateFunction(p1);
		if(fabs(f1) < 0.00001){
			vertex[i][0] = c[0];
			vertex[i][1] = c[1];
			vertex[i][2] = c[2];
			continue;
		}

		float inter = interval;
		if(f1 > 0)
			inter = -inter;
		for(int j=0; j<MAX; j++){
			float d[3];
			func->EvaluateGradient(c, d);
			double len = MeshData::LENGTH(d);
			if(len != 0){
				d[0] /= len;
				d[1] /= len;
				d[2] /= len;
			}
			p2[0] += inter*d[0];
			p2[1] += inter*d[1];
			p2[2] += inter*d[2];
			f2 = func->EvaluateFunction(p2);
			if(f1 * f2 < 0)
				break;
		}
		if(j == MAX){
			vertex[i][0] = c[0];
			vertex[i][1] = c[1];
			vertex[i][2] = c[2];
			continue;
		}

		float p3[3];
		for(j=0; j<MAX; j++){
			p3[0] = 0.5f*(p1[0] + p2[0]);
			p3[1] = 0.5f*(p1[1] + p2[1]);
			p3[2] = 0.5f*(p1[2] + p2[2]);
			float f3 = func->EvaluateFunction(p3);
			if(f3 == 0){
				vertex[i][0] = p3[0];
				vertex[i][1] = p3[1];
				vertex[i][2] = p3[2];
				break;
			}
			else if(f1*f3 >= 0){
				p1[0] = p3[0];
				p1[1] = p3[1];
				p1[2] = p3[2];
				f1 = f3;
			}
			else{
				p2[0] = p3[0];
				p2[1] = p3[1];
				p2[2] = p3[2];
				f2 = f3;
			}
		}
		if(j == MAX){
			vertex[i][0] = p3[0];
			vertex[i][1] = p3[1];
			vertex[i][2] = p3[2];
		}
	}
}

void Smoother::optimizeSquentily(vtkImplicitFunction *func, float T)
{
	int face_N = mesh->face_N;
	int vertex_N = mesh->vertex_N;
	int *degree_f = mesh->degree_f;
	int *degree_v = mesh->degree_v;
	int **f_link = mesh->vertex_link_f;
	int **v_link = mesh->vertex_link_v;
	float (*vertex)[3] = mesh->vertex;
	float (*opt)[3] = new float[vertex_N][3];
	int (*face)[3] = mesh->face;

	//quadlic computation
	double (*Qf)[10] = new double[face_N][10];
	double (*Q)[10] = new double[vertex_N][10];
	BOOL *valid = new BOOL[face_N];
	for(int i=0; i<face_N; i++)
		valid[i] = computeFaceQ(i, func, Qf[i]);
	for(i=0; i<vertex_N; i++){
		int fn = degree_f[i];
		int* link = f_link[i];
		MAT_INIT(Q[i]);
		for(int j=0; j<fn; j++){
			int f = link[j];
			if(valid[f])
				MAT_SUM(Q[i], Qf[f]);
		}
	}

	//optimal position and priorityQ 
	float* err = new float[vertex_N];
	int* heap = new int[vertex_N+1];
	int* index = new int[vertex_N];
	int last_heap = 0;
	for(i=0; i<vertex_N; i++){
		computeOptP(Q[i], vertex[i], opt[i], 0.01);

		float current_err = Q_ERR(Q[i], vertex[i]);
		if(current_err < T){
			index[i] = -1;
			continue;
		}
		if(checkZone(i, opt[i])){
			float opt_err = Q_ERR(Q[i], opt[i]);
			if(current_err > opt_err){
				err[i] = - current_err + opt_err;
				//insert;
				heap[++last_heap] = i;
				index[i] = last_heap;
				upheap(err, last_heap, last_heap, heap, index);
			}
			else{
				index[i] = -1;
			}
		}
		else{
			double L[3];
			mesh->laplacian(i, L);
			vertex[i][0] += L[0];
			vertex[i][1] += L[1];
			vertex[i][2] += L[2];
			index[i] = -1;
		}
	}

	int counter = 0;
	while(last_heap != 0 && counter++ < 4*vertex_N){
		int min = heap[1];
		index[min] = -1;
		//remove
		heap[1] = heap[last_heap--];
		index[heap[1]] = 1;
		downheap(err, last_heap, 1, heap, index);

		//move vertex position
		vertex[min][0] = opt[min][0];
		vertex[min][1] = opt[min][1];
		vertex[min][2] = opt[min][2];

		if(last_heap == 0)
			break;

		//update adjacent Q
		int fn = degree_f[min];
		int *link = f_link[min];
		for(i=0; i<fn; i++)
			valid[link[i]] = computeFaceQ(link[i], func, Qf[link[i]]);

		int vn = degree_v[min];
		int *link1 = v_link[min];
		for(i=0; i<vn; i++){
			int ad = link1[i];
			fn = degree_f[ad];
			link = f_link[ad];
			MAT_INIT(Q[ad]);
			for(int j=0; j<fn; j++){
				int f = link[j];
				if(valid[f])
					MAT_SUM(Q[ad], Qf[f]);
			}

			computeOptP(Q[ad], vertex[ad], opt[ad], 0.01);

			float current_err = Q_ERR(Q[ad], vertex[ad]);
			if(current_err < T){
				if(index[ad] > 0){
					//delete 
					err[ad] = -1;
					upheap(err, last_heap, index[ad], heap, index);
					index[ad] = -1;
					heap[1] = heap[last_heap--];
					if(last_heap < 0)
						break;
					index[heap[1]] = 1;
					downheap(err, last_heap, 1, heap, index);
				}
				continue;
			}

			if(checkZone(ad, opt[ad])){
				float opt_err = Q_ERR(Q[ad], opt[ad]);
				if(current_err > opt_err){
					err[ad] = - current_err + opt_err;
					if(index[ad] < 0){
						//insert
						heap[++last_heap] = ad;
						index[ad] = last_heap;
						upheap(err, last_heap, last_heap, heap, index);
					}
					else{
						//change;
						if(index[ad] != 1 && err[ad] < err[heap[index[ad]/2]])
							upheap(err, last_heap, index[ad], heap, index);
						else
							downheap(err, last_heap, index[ad], heap, index);
					}
				}
				else{
					if(index[ad] > 0){
						//delete
						err[ad] = -1;
						upheap(err, last_heap, index[ad], heap, index);
						index[ad] = -1;
						heap[1] = heap[last_heap--];
						if(last_heap < 0)
							break;
						index[heap[1]] = 1;
						downheap(err, last_heap, 1, heap, index);
					}
				}
			}
			else{
				if(index[ad] > 0){
					err[ad] = -1;
					upheap(err, last_heap, index[ad], heap, index);
					index[ad] = -1;
					heap[1] = heap[last_heap--];
					if(last_heap < 0)
						break;
					index[heap[1]] = 1;
					downheap(err, last_heap, 1, heap, index);
				}
				double L[3];
				mesh->laplacian(ad, L);
				vertex[ad][0] += L[0];
				vertex[ad][1] += L[1];
				vertex[ad][2] += L[2];
			}
		}
	}

	delete[] opt;
	delete[] Q;
	delete[] Qf;
	delete[] index;
	delete[] heap;
	delete[] err;
	delete[] valid;
}

BOOL Smoother::searchNearPoint(vtkImplicitFunction* func, float start[], float p[], float interval, int MAX)
{
	float c[3];
	c[0] = start[0];
	c[1] = start[1];
	c[2] = start[2];
	float p1[3], p2[3], f1, f2;
	p1[0] = p2[0] = c[0];
	p1[1] = p2[1] = c[1];
	p1[2] = p2[2] = c[2];
	f1 = func->EvaluateFunction(p1);
	if(fabs(f1) < 0.001){
		p[0] = c[0];
		p[1] = c[1];
		p[2] = c[2];
		return true;
	}

	float inter = interval;
	if(f1 > 0)
		inter = -inter;
	while(true){
		for(int j=0; j<MAX; j++){
			float d[3];
			func->EvaluateGradient(c, d);
			double len = MeshData::LENGTH(d);
			if(len != 0){
				d[0] /= len;
				d[1] /= len;
				d[2] /= len;
			}
			p2[0] += inter*d[0];
			p2[1] += inter*d[1];
			p2[2] += inter*d[2];
			f2 = func->EvaluateFunction(p2);
			if(f1 * f2 < 0)
				break;
			p1[0] = p2[0];
			p1[1] = p2[1];
			p1[2] = p2[2];
		}
		if(j == MAX){
			inter /= 2.0;
			if(fabs(inter) < 0.00000001)
				return false;
			p1[0] = p2[0] = c[0];
			p1[1] = p2[1] = c[1];
			p1[2] = p2[2] = c[2];
		}
		else
			break;
	}

	//regula falsa
	float p3[3], f3;
	if(f1 > 0.0){
		p3[0] = p1[0];
		p3[1] = p1[1];
		p3[2] = p1[2];

		p1[0] = p2[0];
		p1[1] = p2[1];
		p1[2] = p2[2];

		p2[0] = p3[0];
		p2[1] = p3[1];
		p2[2] = p3[2];

		float swap = f1;
		f1 = f2;
		f2 = swap;
	}

	double dt[3];
	dt[0] = (p2[0] - p1[0]);
	dt[1] = (p2[1] - p1[1]);
	dt[2] = (p2[2] - p1[2]);
	for(int j=0; j<MAX/2; j++){
		if(MeshData::LENGTH(dt) < 0.0001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}

		p3[0] = p1[0] + dt[0]*f1/(f1-f2);
		p3[1] = p1[1] + dt[1]*f1/(f1-f2);
		p3[2] = p1[2] + dt[2]*f1/(f1-f2);

		f3 = func->EvaluateFunction(p3);

		if(fabs(f3) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		if(f3 < 0.0){
			p1[0] = p3[0];
			p1[1] = p3[1];
			p1[2] = p3[2];

			f1 = f3;
		}
		else{
			p2[0] = p3[0];
			p2[1] = p3[1];
			p2[2] = p3[2];

			f2 = f3;
		}
		dt[0] = (p2[0] - p1[0]);
		dt[1] = (p2[1] - p1[1]);
		dt[2] = (p2[2] - p1[2]);
	}

	/*
	//Secant method
	float p3[3];
	if(fabs(f1) < fabs(f2)){
		p3[0] = p1[0];
		p3[1] = p1[1];
		p3[2] = p1[2];

		p1[0] = p2[0];
		p1[1] = p2[1];
		p1[2] = p2[2];

		float swap = f1;
		f1 = f2;
		f2 = swap;
	}
	else{
		p3[0] = p2[0];
		p3[1] = p2[1];
		p3[2] = p2[2];
	}
	for(int j=0; j<MAX; j++){
		float dt[3];
		dt[0] = (p1[0] - p3[0])*f2/(f2-f1);
		dt[1] = (p1[1] - p3[1])*f2/(f2-f1);
		dt[2] = (p1[2] - p3[2])*f2/(f2-f1);

		if(MeshData::LENGTH(dt) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		
		p1[0] = p3[0];
		p1[1] = p3[1];
		p1[2] = p3[2];

		f1 = f2;

		p3[0] += dt[0];
		p3[1] += dt[1];
		p3[2] += dt[2];

		f2 = func->EvaluateFunction(p3);

		if(fabs(f2) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
	}*/

	//Bisection method
	//float p3[3];
	for(j=0; j<2.0*MAX; j++){
		if(MeshData::DIST(p1, p2) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}

		p3[0] = 0.5f*(p1[0] + p2[0]);
		p3[1] = 0.5f*(p1[1] + p2[1]);
		p3[2] = 0.5f*(p1[2] + p2[2]);
		f3 = func->EvaluateFunction(p3);
		if(fabs(f3) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		else if(f1*f3 >= 0){
			p1[0] = p3[0];
			p1[1] = p3[1];
			p1[2] = p3[2];
			f1 = f3;
		}
		else{
			p2[0] = p3[0];
			p2[1] = p3[1];
			p2[2] = p3[2];
			f2 = f3;
		}
	}

	/*
	p[0] = p3[0];
	p[1] = p3[1];
	p[2] = p3[2];

	if(fabs(func->EvaluateFunction(p)) < 0.001)
		return true;
		
	else*/
	p[0] = start[0];
	p[1] = start[1];
	p[2] = start[2];
	return false;
}

BOOL Smoother::computeFaceQ(int i, vtkImplicitFunction *func, double Q[])
{
	double area = mesh->faceArea(i);

	float c[3], p[3];
	mesh->faceCenter(c, i);
	if(!searchNearPoint(func, c, p, sqrt(area), 40))
		return false;

	float g[3];
	func->EvaluateGradient(p, g);
	double len = MeshData::LENGTH(g);
	if(len == 0)
		return false;

	double n[3];
	n[0] = g[0]/len;
	n[1] = g[1]/len;
	n[2] = g[2]/len;
		
	double d = -MeshData::DOT(n, p);
	MATRIX(Q, n, d);
	MAT_TIMES(Q, area);
	
	return true;
}

BOOL Smoother::computeOptP(double Q[], float p[], float opt[], float C)
{
	/*
	Q[0] += C;
	Q[3] += C;
	Q[5] += C;
	Q[6] -= C*p[0];
	Q[7] -= C*p[1];
	Q[8] -= C*p[2];
	Q[9] += C*MeshData::DOT(p, p);
	*/
	double inv[10];
	double b[3];
	b[0] = -Q[6];
	b[1] = -Q[7];
	b[2] = -Q[8];

	double opt_d[3];
	if(INVERSE(inv, Q))
		MAT_BY_VEC(opt_d, inv, b);
	else{
		opt[0] = p[0];
		opt[1] = p[1];
		opt[2] = p[2];
		return false;
	}
	/*
	else{
		double Q1[10];
		MAT_COPY(Q1, Q);

		Q1[0] += C;
		Q1[3] += C;
		Q1[5] += C;
		if(INVERSE(inv, Q1)){
			b[0] += C*p[0];
			b[1] += C*p[1];
			b[2] += C*p[2];
			MAT_BY_VEC(opt_d, inv, b);
		}
		else
			return false;
	}*/
	opt[0] = opt_d[0];
	opt[1] = opt_d[1];
	opt[2] = opt_d[2];
	return true;
}

BOOL Smoother::checkZone(int i, float opt[])
{
	int *link = mesh->vertex_link_f[i];
	int fn = mesh->degree_f[i];
	float (*vertex)[3] = mesh->vertex;

	for(int j=0; j<fn; j++){
		int f = link[j];
		int *face = mesh->face[f];

		int i1, i2;
		if(face[0] == i){
			i1 = face[1];
			i2 = face[2];
		}
		else if(face[1] == i){
			i1 = face[2];
			i2 = face[0];
		}
		else{
			i1 = face[0];
			i2 = face[1];
		}

		float v[3];
		v[0] = vertex[i1][0] - vertex[i2][0];
		v[1] = vertex[i1][1] - vertex[i2][1];
		v[2] = vertex[i1][2] - vertex[i2][2];

		float v1[3], v2[3];
		MeshData::VEC(v1, vertex[i], vertex[i1]);
		MeshData::VEC(v2, vertex[i], vertex[i2]);
		double m[3];
		MeshData::CROSS(m, v1, v2);

		double n[3];
		MeshData::CROSS(n, v, m);
		
		float e[3];
		e[0] = 0.5*(vertex[i1][0] + vertex[i2][0]);
		e[1] = 0.5*(vertex[i1][1] + vertex[i2][1]);
		e[2] = 0.5*(vertex[i1][2] + vertex[i2][2]);

		MeshData::VEC(v, vertex[i], e);
		double side1 = MeshData::DOT(n, v);

		MeshData::VEC(v, opt, e);
		double side2 = MeshData::DOT(n, v);

		if(side1 * side2 <= 0)
			return false;

		if(SHAPE(vertex[i], vertex[i1], vertex[i2]) < 0.01)
			return false;
	}
	return true;
}

void Smoother::setCenterOnSurface(vtkImplicitFunction *func)
{
	int face_N = mesh->face_N;
	float (*center)[3] = mesh->center;
	BOOL* valid = mesh->valid_f = new BOOL[face_N];

	for(int i=0; i<face_N; i++){
		float p[3];
		double area = mesh->triangleEdgeAverage(i); //mesh->faceArea(i);
		valid[i] = searchNearPoint(func, center[i], p, 0.5*area, 20);
		//double area = mesh->faceArea(i);
		//valid[i] = searchNearPoint(func, center[i], p, sqrt(area), 20);
		/***********/
		float c1[3];
		c1[0] = 2*center[i][0] - p[0];
		c1[1] = 2*center[i][1] - p[1];
		c1[2] = 2*center[i][2] - p[2];
		if((func->EvaluateFunction(center[i]))*(func->EvaluateFunction(c1)) < 0){
			float d[3];
			d[0] = center[i][0] - p[0];
			d[1] = center[i][1] - p[1];
			d[2] = center[i][2] - p[2];
			double l = MeshData::LENGTH(d);
			if(d != 0){
				d[0] /= -l;
				d[1] /= -l;
				d[2] /= -l;
				valid[i] = searchNearPoint1D(func, center[i], c1, d, 0.5*area, 20);
				if(MeshData::DIST(center[i], c1) < MeshData::DIST(center[i], p)){
					p[0] = c1[0];
					p[1] = c1[1];
					p[2] = c1[2];
				}
			}
		}
		/***********/
		if(MeshData::DIST(center[i], p) > area){
			valid[i] = false;
		}
		else{
			center[i][0] = p[0];
			center[i][1] = p[1];
			center[i][2] = p[2];
		}
	}
/*
	for(int i=0; i<face_N; i++){
		func->EvaluateGradient(center[i], n[i]);
		double len = MeshData::LENGTH(n[i]);
		if(len != 0){
			n[i][0] /= len;
			n[i][1] /= len;
			n[i][2] /= len;
		}
	}*/
}

void Smoother::uniformizeCenter(float C)
{
	int (*link)[3] = mesh->face_link_E;
	int face_N = mesh->face_N;
	float (*dc)[3] = new float[face_N][3];
	for(int i=0; i<face_N; i++){
		int* ad = link[i];
		float U[3];
		/*
		U[0] = (center[ad[0]][0] + center[ad[1]][0] + center[ad[2]][0])/3.0f - center[i][0];
		U[1] = (center[ad[0]][1] + center[ad[1]][1] + center[ad[2]][1])/3.0f - center[i][1];
		U[2] = (center[ad[0]][2] + center[ad[1]][2] + center[ad[2]][2])/3.0f - center[i][2];

		double dot = MeshData::DOT(m[i], U);
		*/

		U[0] = U[1] = U[2] = 0;
		float total_w = 0;
		for(int j=0; j<3; j++){
			double dot = MeshData::DOT(m[i], m[ad[j]]);
			if(dot > 1.0)
				dot = 1.0;
			else if(dot < -1)
				dot = -1.0;
			double d = MeshData::DIST(center[i], center[ad[j]]);
			double w;
			if(d == 0)
				w = 0.1;
			else 
			    w = 0.1 + acos(dot)/d;
			total_w += w;
			U[0] += w*(center[ad[j]][0] - center[i][0]);
			U[1] += w*(center[ad[j]][1] - center[i][1]);
			U[2] += w*(center[ad[j]][2] - center[i][2]);
		}
		U[0] /= total_w;
		U[1] /= total_w;
		U[2] /= total_w;

		double dot = MeshData::DOT(m[i], U);

		dc[i][0] = U[0]; // - dot*m[i][0];
		dc[i][1] = U[1]; // - dot*m[i][1];
		dc[i][2] = U[2]; // - dot*m[i][2];
	}
	for(i=0; i<face_N; i++){
		center[i][0] += C*dc[i][0];
		center[i][1] += C*dc[i][1];
		center[i][2] += C*dc[i][2];
	}
	delete[] dc;
}

void Smoother::fillValid()
{
	int (*link)[3] = mesh->face_link_E;
	int face_N = mesh->face_N;
	BOOL allOK = false;
	while(!allOK){
		allOK = true;
		for(int i=0; i<face_N; i++){
			if(!valid[i]){
				allOK = false;
				int* ad = link[i];

				center[i][0] = center[i][1] = center[i][2] = 0;
				int valid_N = 0;
				for(int j=0; j<3; j++){
					if(!valid[ad[j]])
						continue;
					valid_N++;
					center[i][0] += center[ad[j]][0];
					center[i][1] += center[ad[j]][1];
					center[i][2] += center[ad[j]][2];
				}
				if(valid_N != 0){
					center[i][0] /= valid_N;
					center[i][1] /= valid_N;
					center[i][2] /= valid_N;
					valid[i] = true;
				}
			}
		}
	}
}

void Smoother::setVertexFromCenterW(float T)
{
	int face_N = mesh->face_N;
	double *w = new double[face_N];
	int (*link_f)[3] = mesh->face_link_E;

	for(int i=0; i<face_N; i++)
		w[i] = 0;

	for(i=0; i<face_N; i++){
		if(!valid[i])
			continue;
		int* ad = link_f[i];
		for(int j=0; j<3; j++){
			if(ad[j] > i)
				continue;
			if(!valid[ad[j]])
				continue;
			double dot = MeshData::DOT(m[i], m[ad[j]]);
			if(dot > 1.0)
				dot = 1.0;
			else if(dot < -1.0)
				dot = -1.0;
			double d = MeshData::DIST(center[i], center[ad[j]]);
			if(d == 0)
				continue;
			double k = acos(dot)/d;
			w[i] += k;
			w[ad[j]] += k;
		}
	}

	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int *degree = mesh->degree_f;
	int **link = mesh->vertex_link_f;

	for(i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		float p[3];
		p[0] = p[1] = p[2] = 0;
		double total_w = 0;
		for(int j=0; j<deg; j++){
			if(!valid[l[j]])
				continue;
			int f = l[j];
			double wf = (1.0 + T*w[f]);
			total_w += wf;
			p[0] += wf*center[f][0];
			p[1] += wf*center[f][1];
			p[2] += wf*center[f][2];
		}
		if(total_w != 0){
			vertex[i][0] = p[0]/total_w;
			vertex[i][1] = p[1]/total_w;
			vertex[i][2] = p[2]/total_w;
		}
	}

	delete[] w;
}

void Smoother::resolveFlip()
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f;
	float (*normal_f)[3] = mesh->normal_f;
	float (*normal)[3] = mesh->normal;
	int (*face)[3] = mesh->face;

	BOOL allOK = false;

	while(!allOK){
		allOK = true;
		for(int i=0; i<vertex_N; i++){
			int *l = link[i];
			int deg = degree[i];
			double (*n)[3] = new double[deg][3];
			double ni[3];
			ni[0] = ni[1] = ni[2] = 0;
			for(int j=0; j<deg; j++){
				int f1 = l[j];
				int f2 = l[(j+1)%deg];
				float v1[3], v2[3];
				MeshData::VEC(v1, vertex[i], center[f1]);
				MeshData::VEC(v2, vertex[i], center[f2]);
				MeshData::CROSS(n[j], v1, v2);
				ni[0] += n[j][0];
				ni[1] += n[j][1];
				ni[2] += n[j][2];
			}
			double len = MeshData::LENGTH(ni);
			if(len == 0)
				continue;
			ni[0] /= len;
			ni[1] /= len;
			ni[2] /= len;
			for(j=0; j<deg; j++){
				len = MeshData::LENGTH(n[j]);
				if(len == 0)
					continue;
				if(MeshData::DOT(ni, n[j])/len < -0.9){
					int f1 = l[j];
					int f2 = l[(j+1)%deg];
			
					float temp[3];
					temp[0] = center[f1][0];
					temp[1] = center[f1][1];
					temp[2] = center[f1][2];
					center[f1][0] = center[f2][0];
					center[f1][1] = center[f2][1];
					center[f1][2] = center[f2][2];
					center[f2][0] = temp[0];
					center[f2][1] = temp[1];
					center[f2][2] = temp[2];
					allOK = false;
				}
			}
			delete[] n;
		}
	}
}

void Smoother::computeQuadric(double (*Q)[10])
{
	int face_N = mesh->face_N;
	int vertex_N = mesh->vertex_N;
	int **f_link = mesh->vertex_link_f;
	int *degree_f = mesh->degree_f;
	double (*Qf)[10] = new double[face_N][10];

	for(int i=0; i<face_N; i++){
		double mi[3];
		mi[0] = m[i][0];
		mi[1] = m[i][1];
		mi[2] = m[i][2];
		//double w = mesh->faceArea(i);
		double d = -MeshData::DOT(mi, center[i]);
		MATRIX(Qf[i], mi, d);
		//MAT_TIMES(Qf[i], w);
	}

	for(i=0; i<vertex_N; i++){
		int fn = degree_f[i];
		int* link = f_link[i];
		double total_w = 0;
		MAT_INIT(Q[i]);
		for(int j=0; j<fn; j++){
			if(valid[link[j]]){
				total_w += 1; //mesh->faceArea(link[j]);
				MAT_SUM(Q[i], Qf[link[j]]);
			}
		}
		if(total_w != 0)
			MAT_TIMES(Q[i], 1.0/total_w);
	}

	delete[] Qf;
}

void Smoother::minimizeQ(int iter, double (*Q)[10])
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	for(int i=0; i<vertex_N; i++){
		this->computeOptP(Q[i], vertex[i], vertex[i], 0);
		/*
		double v[3];
		v[0] = vertex[i][0];
		v[1] = vertex[i][1];
		v[2] = vertex[i][2];
		double b[3];
		b[0] = Q[i][6];
		b[1] = Q[i][7];
		b[2] = Q[i][8];
		for(int j=0; j<iter; j++){
			double Mv[3];
			MAT_BY_VEC(Mv, Q[i], v);
			v[0] -= Mv[0] + b[0];
			v[1] -= Mv[1] + b[1];
			v[2] -= Mv[2] + b[2];
			if(Q_ERR(Q[i], v) < 0.000001)
				break;
		}
		vertex[i][0] = v[0];
		vertex[i][1] = v[1];
		vertex[i][2] = v[2]; 
		*/
	}
}

void Smoother::averagingQ(int iter, double (*Q)[10])
{

	int vertex_N = mesh->vertex_N;
	int **link = mesh->vertex_link_v;
	int *degree = mesh->degree_v;
	double (*tempQ)[10] = new double[vertex_N][10];
	
	for(int k=0; k<iter; k++){
		for(int i=0; i<vertex_N; i++){
			int deg = degree[i];
			int *l = link[i];
			MAT_INIT(tempQ[i]);
			for(int j=0; j<deg; j++){
				MAT_PLUS(tempQ[i], tempQ[i], Q[l[j]]);
			}
			double lQ[10];
			MAT_COPY(lQ, Q[i]);
			MAT_TIMES(lQ, 0.5);
			MAT_TIMES(tempQ[i], 0.5/deg);
			MAT_PLUS(tempQ[i], tempQ[i], lQ);
		}
		for(i=0; i<vertex_N; i++)
			MAT_COPY(Q[i], tempQ[i]);
	}
	/*
	double dt = 0.5;
	double dt1 = dt;
	double dt2 = dt1/(-1+dt1*0.1);
	for(int k=0; k<iter; k++){
		if(k%2 == 0)
			dt = dt1;
		else
			dt = dt2;
		for(int i=0; i<vertex_N; i++){
			int deg = degree[i];
			int *l = link[i];
			MAT_INIT(tempQ[i]);
			for(int j=0; j<deg; j++){
				MAT_PLUS(tempQ[i], tempQ[i], Q[l[j]]);
			}
			double lQ[10];
			MAT_COPY(lQ, Q[i]);
			MAT_TIMES(lQ, (1.0-dt));
			MAT_TIMES(tempQ[i], dt/deg);
			MAT_PLUS(tempQ[i], tempQ[i], lQ);
		}
		for(i=0; i<vertex_N; i++)
			MAT_COPY(Q[i], tempQ[i]);
	}
	*/
	delete[] tempQ;
}

void Smoother::checkFlip(vtkImplicitFunction *func)
{
	mesh->computeFaceNormal();
	float (*normal)[3] = mesh->normal_f;
	int face_N = mesh->face_N;
	mesh->computeCenter();

	int count = 0;
	for(int i=0; i<face_N; i++){
		float c[3];
		mesh->faceCenter(c, i);
		float g[3];
		func->EvaluateGradient(c, g);
		double len = MeshData::LENGTH(g);
		if(len == 0)
			continue;
		if(MeshData::DOT(normal[i], g)/len > 0.9)
			count++;
		else if(mesh->faceArea(i) < 0.00000001)
			count++;
	}
}

void Smoother::generateUniformCenter(vtkImplicitFunction *func, int iter, float W)
{
	int i;

	int face_N = mesh->face_N;
	if(center == NULL){
		center = new float[face_N][3];
		valid = new BOOL[face_N];
		m = new float[face_N][3];
	}
	
	for(i=0; i<iter; i++){
		for(int j=0; j<face_N; j++)
			mesh->faceCenter(center[j], j);
		//setCenterNormal(func);
		computeCenterBisection(0.1, func, 20);
		setCenterNormal(func);
		if(W != 0 && i < iter -1)
			setVertexFromCenterW(W);
		else
			setVertexFromCenter();
	}
	//setCenterNormal(func);
}

void Smoother::replaceOptimalPosition(int MAX, float uni_w, int ave)
{
	//setVertexFromCenter();

	if(center == NULL)
		return;

	double (*Q)[10] = new double[mesh->vertex_N][10];
	computeQuadric(Q);

	/*
	if(uni_w != 0){
		double (*dir)[3] = new double[mesh->vertex_N][3];
		computeDir(dir);
		addUniformW(Q, dir, 10*uni_w);
		delete[] dir;
	}

	if(ave != 0)
		averagingQ(ave, Q);
	//minimizeQ(MAX, Q);
	minimizeQPowell(Q, 0.00001);
	computeQuadric(Q);
	*/

	if(uni_w != 0){
		double (*dir)[3] = new double[mesh->vertex_N][3];
		computeDir(dir);
		//addVolumeW(Q, uni_w);
		addUniformW(Q, dir, uni_w);
		//addDerivativeError(Q, uni_w);
		delete[] dir;
	}

	if(ave != 0)
		averagingQ(ave, Q);
	//minimizeH1(MAX, Q, -uni_w);
	minimizeQ(MAX, Q);
	//minimizeQPowell(Q, 0.00001);
	
	delete[] Q;
}

void Smoother::minimizeQ1D(double (*Q)[10], double (*dir)[3])
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;

	for(int i=0; i<vertex_N; i++){
		double *A = Q[i];
		double *d = dir[i];
		float *p = vertex[i];

		double over = (-A[6]-A[0]*p[0] - A[1]*p[1] - A[2]*p[2])*d[0] +
					  (-A[7]-A[1]*p[0] - A[3]*p[1] - A[4]*p[2])*d[1] +
					  (-A[8]-A[2]*p[0] - A[4]*p[1] - A[5]*p[2])*d[2] ;
		double under = (A[0]*d[0] + A[1]*d[1] + A[2]*d[2])*d[0] + 
					   (A[1]*d[0] + A[3]*d[1] + A[4]*d[2])*d[1] + 
					   (A[2]*d[0] + A[4]*d[1] + A[5]*d[2])*d[2] ;
		if(under != 0){
			double t = over/under;
			vertex[i][0] += t*d[0];
			vertex[i][1] += t*d[1];
			vertex[i][2] += t*d[2];
		}
	}
}

void Smoother::computeDir(double (*dir)[3])
{
	int vertex_N = mesh->vertex_N;
	int face_N = mesh->face_N;
	int (*face)[3] = mesh->face;

	for(int i=0; i<vertex_N; i++)
		dir[i][0] = dir[i][1] = dir[i][2] = 0;

	for(i=0; i<face_N; i++){
		for(int j=0; j<3; j++){
			int k = face[i][j];
			if(!valid[k])
				continue;
			dir[k][0] += m[i][0];
			dir[k][1] += m[i][1];
			dir[k][2] += m[i][2];
		}
	}
	for(i=0; i<vertex_N; i++){
		double l = MeshData::LENGTH(dir[i]);
		if(l != 0){
			dir[i][0] /= l;
			dir[i][1] /= l;
			dir[i][2] /= l;
		}
	}
}

void Smoother::addUniformW(double (*Q)[10], double (*dir)[3], float C)
{
	int vertex_N = mesh->vertex_N;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f;

	for(int i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		double Q1[10];
		MAT_INIT(Q1);
		int valid_N = 0;
		for(int j=0; j<deg; j++){
			int f = l[j];
			if(!valid[f])
				continue;
			float v[3];
			MeshData::VEC(v, center[f], center[l[(j+1)%deg]]);
			double n[3];
			MeshData::CROSS(n, v, dir[i]);
			
			double len = MeshData::LENGTH(n);
			if(len == 0)
				continue;
			n[0] /= len;
			n[1] /= len;
			n[2] /= len;

			double d = -MeshData::DOT(n, center[f]);
			double Qf[10];
			MATRIX(Qf, n, d);
			MAT_SUM(Q1, Qf);
			valid_N++;
		}
		if(valid_N != 0){
			MAT_TIMES(Q1, C/valid_N);
			MAT_SUM(Q[i], Q1);
		}
	}
}

void Smoother::minimizeQPowell(double (*Q)[10], float T)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;


	LocalTangentError e;
	EnergyMinimizer mini;
	mini.energy = &e;

	for(int i=0; i< vertex_N; i++){
		e.Q = Q[i];

		int iter;
		float fret;
		fret = e.func(vertex[i]);
		//for(int k=0; k<10; k++){
			mini.frprmn(vertex[i], 3, T, &iter, &fret);
			//if(fret < 0.0000001)
				//break;
		//}
	}
}

void Smoother::addUniformW2(double (*Q)[10], double (*dir)[3], float C)
{
	int vertex_N = mesh->vertex_N;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f;
	int **link_v= mesh->vertex_link_v;

	for(int i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		int *lv = link_v[i];
		double Q1[10];
		MAT_INIT(Q1);
		int valid_N = 0;
		for(int j=0; j<deg; j++){
			int f = l[j];
			int pair = lv[j];
			double de[3];
			de[0] = dir[i][0] + dir[pair][0];
			de[1] = dir[i][1] + dir[pair][1];
			de[2] = dir[i][2] + dir[pair][2];

			if(!valid[f])
				continue;
			float v[3];
			MeshData::VEC(v, center[f], center[l[(j+1)%deg]]);
			double n[3];
			MeshData::CROSS(n, v, de);
			double len = MeshData::LENGTH(n);
			if(len == 0)
				continue;
			n[0] /= len;
			n[1] /= len;
			n[2] /= len;

			double d = -MeshData::DOT(n, center[f]);
			double Qf[10];
			MATRIX(Qf, n, d);
			MAT_SUM(Q1, Qf);
			valid_N++;
		}
		if(valid_N != 0){
			MAT_TIMES(Q1, C);
			MAT_SUM(Q[i], Q1);
		}
	}
}

void Smoother::addDerivativeError(double (*Q)[10], float C)
{
	int vertex_N = mesh->vertex_N;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f;

	for(int i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		double deri[] = {0,0,0,0};
		int valid_N = 0;
		for(int j=0; j<deg; j++){
			int f1 = l[j];
			int f2 = l[(j+1)%deg];
			if(!valid[f1] || !valid[f2])
				continue;

			double n[3];
			n[0] = m[f1][0] + m[f2][0];
			n[1] = m[f1][1] + m[f2][1];
			n[2] = m[f1][2] + m[f2][2];
			double l = MeshData::LENGTH(n);
			if(l == 0)
				continue;
			n[0] /= l;
			n[1] /= l;
			n[2] /= l;

			float *p1 = center[f1];
			float *p2 = center[f2];

			deri[0] += n[1]*(p2[2] - p1[2]) + n[2]*(p1[1] - p2[1]);
			deri[1] += n[2]*(p2[0] - p1[0]) + n[0]*(p1[2] - p2[2]);
			deri[2] += n[0]*(p2[1] - p1[1]) + n[1]*(p1[0] - p2[0]);
			deri[3] += n[0]*(p1[1]*p2[2] - p1[2]*p2[1]) 
				    +  n[1]*(p1[2]*p2[0] - p1[0]*p2[2]) 
				    +  n[2]*(p1[0]*p2[1] - p1[1]*p2[0]);

			valid_N++;
		}
		if(valid_N != 0){
			Q[i][6] += C*deri[0]/valid_N;
			Q[i][7] += C*deri[1]/valid_N;
			Q[i][8] += C*deri[2]/valid_N;
			Q[i][9] += C*deri[3]/valid_N;
		}
	}
}

void Smoother::minimizeH1(int iter, double (*Q)[10], float C)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f; 

	for(int i=0; i<vertex_N; i++){	
		double v[3];
		v[0] = vertex[i][0];
		v[1] = vertex[i][1];
		v[2] = vertex[i][2];
		double b[3];
		b[0] = Q[i][6];
		b[1] = Q[i][7];
		b[2] = Q[i][8];

		int *l = link[i];
		int deg = degree[i];
		float (*mi)[3] = new float[deg][3];
		for(int j=0; j<deg; j++){
			int f1 = l[j];
			int f2 = l[(j+1)%deg];
			if(!valid[f1] || !valid[f2])
				continue;
			mi[j][0] = m[f1][0] + m[f2][0];
			mi[j][1] = m[f1][1] + m[f2][1];
			mi[j][2] = m[f1][2] + m[f2][2];

			double len = MeshData::LENGTH(mi[j]);
			if(len != 0){
				mi[j][0] /= len;
				mi[j][1] /= len;
				mi[j][2] /= len;
			}
		}

		for(j=0; j<iter; j++){
			double Mv[3];
			double v1[3];
			MAT_BY_VEC(Mv, Q[i], v);
			v1[0] = Mv[0] + b[0];
			v1[1] = Mv[1] + b[1];
			v1[2] = Mv[2] + b[2];
			//if(Q_ERR(Q[i], v) < 0.000001)
				//break;

			float v2[] = {0,0,0};
			int valid_N = 0;
			float en = 0;
			for(int k = 0; k<deg; k++){
				int f1 = l[k];
				int f2 = l[(k+1)%deg];

				if(!valid[f1] || !valid[f2])
					continue;

				double q1[3], q2[3];
				MeshData::VEC(q1, v, center[f1]);
				MeshData::VEC(q2, v, center[f2]);
				double c[3];
				MeshData::CROSS(c, q1, q2);

				double cx[3], cy[3], cz[3];

				cx[0] = 0;
				cx[1] = q1[2] - q2[2];
				cx[2] = q2[1] - q1[1];
			
				cy[0] = q2[2] - q1[2];
				cy[1] = 0;
				cy[2] = q1[0] - q2[0];

				cz[0] = q1[1] - q2[1];
				cz[1] = q2[0] - q1[0];
				cz[2] = 0;

				double c1 = MeshData::LENGTH(c);
				double nm[3];
				if(c1 == 0)
					continue;
				nm[0] = c[0]/c1 - mi[k][0];
				nm[1] = c[1]/c1 - mi[k][1];
				nm[2] = c[2]/c1 - mi[k][2];

				v2[0] += MeshData::DOT(cx, nm);
				v2[1] += MeshData::DOT(cy, nm);
				v2[2] += MeshData::DOT(cz, nm);

				valid_N++;

				en += c1 - MeshData::DOT(mi[k], c);
			}

			//v1[0] = v1[1] = v1[2] = 0;

			v[0] -= v1[0] + C*v2[0]/valid_N;
			v[1] -= v1[1] + C*v2[1]/valid_N;
			v[2] -= v1[2] + C*v2[2]/valid_N;

			if(Q_ERR(Q[i], v) + en < 0.00001)
				break;
		}
		delete [] mi;

		vertex[i][0] = v[0];
		vertex[i][1] = v[1];
		vertex[i][2] = v[2]; 
	}
}

void Smoother::addVolumeW(double (*Q)[10], float C)
{
	int vertex_N = mesh->vertex_N;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f;

	for(int i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		double Q1[10];
		MAT_INIT(Q1);
		int valid_N = 0;
		for(int j=0; j<deg; j++){
			int f1 = l[j];
			int f2 = l[(j+1)%deg];

			if(!valid[f1] || !valid[f2])
				continue;

			float v[3];
			MeshData::VEC(v, center[f1], center[f2]);
			
			double len = MeshData::LENGTH(v);
			if(len == 0)
				continue;
			v[0] /= len;
			v[1] /= len;
			v[2] /= len;

			float dir[3];
			dir[0] = m[f1][0] + m[f2][0];
			dir[1] = m[f1][1] + m[f2][1];
			dir[2] = m[f1][2] + m[f2][2];

			double n[3];
			MeshData::CROSS(n, v, dir);
			
			double d = -MeshData::DOT(n, center[f1]);
			double Qf[10];
			MATRIX(Qf, n, d);
			MAT_SUM(Q1, Qf);
			valid_N++;
		}
		if(valid_N != 0){
			MAT_TIMES(Q1, C/valid_N);
			MAT_SUM(Q[i], Q1);
		}
	}
}

void Smoother::addVolumeW2(double (*Q)[10], float C)
{
	int vertex_N = mesh->vertex_N;
	int **link = mesh->vertex_link_f;
	int *degree = mesh->degree_f;

	for(int i=0; i<vertex_N; i++){
		int deg = degree[i];
		int *l = link[i];
		double Q1[10];
		MAT_INIT(Q1);
		int valid_N = 0;
		for(int j=0; j<deg; j++){
			int f1 = l[j];
			int ff = l[(j+1)%deg];
			int fb = l[(j-1+deg)%deg];

			if(!valid[f1] )
				continue;

			float v[3];
			if(MeshData::DOT(m[ff], m[f1]) > MeshData::DOT(m[fb], m[f1]))
				MeshData::VEC(v, center[f1], center[ff]);
			else
				MeshData::VEC(v, center[fb], center[f1]);
	
			double len = MeshData::LENGTH(v);
			if(len == 0)
				continue;
			v[0] /= len;
			v[1] /= len;
			v[2] /= len;

			double n[3];
			MeshData::CROSS(n, v, m[f1]);
			
			double d = -MeshData::DOT(n, center[f1]);
			double Qf[10];
			MATRIX(Qf, n, d);
			MAT_SUM(Q1, Qf);
			valid_N++;
		}
		if(valid_N != 0){
			MAT_TIMES(Q1, C/valid_N);
			MAT_SUM(Q[i], Q1);
		}
	}
}

BOOL Smoother::searchNearPoint1D(vtkImplicitFunction *func, float start[], float p[], float *d, float interval, int MAX)
{
		float c[3];
	c[0] = start[0];
	c[1] = start[1];
	c[2] = start[2];
	float p1[3], p2[3], f1, f2;
	p1[0] = p2[0] = c[0];
	p1[1] = p2[1] = c[1];
	p1[2] = p2[2] = c[2];
	f1 = func->EvaluateFunction(p1);
	if(fabs(f1) < 0.001){
		p[0] = c[0];
		p[1] = c[1];
		p[2] = c[2];
		return true;
	}

	float inter = interval;
	if(f1 > 0)
		inter = -inter;
	while(true){
		for(int j=0; j<MAX; j++){
			/*
			float d[3];
			func->EvaluateGradient(c, d);
			double len = MeshData::LENGTH(d);
			if(len != 0){
				d[0] /= len;
				d[1] /= len;
				d[2] /= len;
			}
			*/
			p2[0] += inter*d[0];
			p2[1] += inter*d[1];
			p2[2] += inter*d[2];
			f2 = func->EvaluateFunction(p2);
			if(f1 * f2 < 0)
				break;
			p1[0] = p2[0];
			p1[1] = p2[1];
			p1[2] = p2[2];
		}
		if(j == MAX){
			inter /= 2.0;
			if(fabs(inter) < 0.00000001)
				return false;
			p1[0] = p2[0] = c[0];
			p1[1] = p2[1] = c[1];
			p1[2] = p2[2] = c[2];
		}
		else
			break;
	}

	//regula falsa
	float p3[3], f3;
	if(f1 > 0.0){
		p3[0] = p1[0];
		p3[1] = p1[1];
		p3[2] = p1[2];

		p1[0] = p2[0];
		p1[1] = p2[1];
		p1[2] = p2[2];

		p2[0] = p3[0];
		p2[1] = p3[1];
		p2[2] = p3[2];

		float swap = f1;
		f1 = f2;
		f2 = swap;
	}

	double dt[3];
	dt[0] = (p2[0] - p1[0]);
	dt[1] = (p2[1] - p1[1]);
	dt[2] = (p2[2] - p1[2]);
	for(int j=0; j<MAX/2; j++){
		if(MeshData::LENGTH(dt) < 0.0001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}

		p3[0] = p1[0] + dt[0]*f1/(f1-f2);
		p3[1] = p1[1] + dt[1]*f1/(f1-f2);
		p3[2] = p1[2] + dt[2]*f1/(f1-f2);

		f3 = func->EvaluateFunction(p3);

		if(fabs(f3) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		if(f3 < 0.0){
			p1[0] = p3[0];
			p1[1] = p3[1];
			p1[2] = p3[2];

			f1 = f3;
		}
		else{
			p2[0] = p3[0];
			p2[1] = p3[1];
			p2[2] = p3[2];

			f2 = f3;
		}
		dt[0] = (p2[0] - p1[0]);
		dt[1] = (p2[1] - p1[1]);
		dt[2] = (p2[2] - p1[2]);
	}

	//Bisection method
	for(j=0; j<2.0*MAX; j++){
		if(MeshData::DIST(p1, p2) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}

		p3[0] = 0.5f*(p1[0] + p2[0]);
		p3[1] = 0.5f*(p1[1] + p2[1]);
		p3[2] = 0.5f*(p1[2] + p2[2]);
		f3 = func->EvaluateFunction(p3);
		if(fabs(f3) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		else if(f1*f3 >= 0){
			p1[0] = p3[0];
			p1[1] = p3[1];
			p1[2] = p3[2];
			f1 = f3;
		}
		else{
			p2[0] = p3[0];
			p2[1] = p3[1];
			p2[2] = p3[2];
			f2 = f3;
		}
	}
	return false;
}

BOOL Smoother::searchNearPoint2dire(vtkImplicitFunction *func, float start[], float p[], float interval, int MAX)
{
	float c[3];
	c[0] = start[0];
	c[1] = start[1];
	c[2] = start[2];
	float p1[3], p2[3], f1, f2;
	p1[0] = p2[0] = c[0];
	p1[1] = p2[1] = c[1];
	p1[2] = p2[2] = c[2];
	f1 = func->EvaluateFunction(p1);
	if(fabs(f1) < 0.001){
		p[0] = c[0];
		p[1] = c[1];
		p[2] = c[2];
		return true;
	}

	float inter = interval;
	if(f1 > 0)
		inter = -inter;
	while(true){
		for(int j=0; j<MAX; j++){
			float d[3];
			func->EvaluateGradient(c, d);
			double len = MeshData::LENGTH(d);
			if(len != 0){
				d[0] /= len;
				d[1] /= len;
				d[2] /= len;
			}
			p2[0] += inter*d[0];
			p2[1] += inter*d[1];
			p2[2] += inter*d[2];
			f2 = func->EvaluateFunction(p2);
			if(f1 * f2 < 0)
				break;
			p1[0] = p2[0];
			p1[1] = p2[1];
			p1[2] = p2[2];
		}
		if(j == MAX){
			inter /= 2.0;
			if(fabs(inter) < 0.00000001)
				return false;
			p1[0] = p2[0] = c[0];
			p1[1] = p2[1] = c[1];
			p1[2] = p2[2] = c[2];
		}
		else
			break;
	}

	//regula falsa
	float p3[3], f3;
	if(f1 > 0.0){
		p3[0] = p1[0];
		p3[1] = p1[1];
		p3[2] = p1[2];

		p1[0] = p2[0];
		p1[1] = p2[1];
		p1[2] = p2[2];

		p2[0] = p3[0];
		p2[1] = p3[1];
		p2[2] = p3[2];

		float swap = f1;
		f1 = f2;
		f2 = swap;
	}

	double dt[3];
	dt[0] = (p2[0] - p1[0]);
	dt[1] = (p2[1] - p1[1]);
	dt[2] = (p2[2] - p1[2]);
	for(int j=0; j<MAX/2; j++){
		if(MeshData::LENGTH(dt) < 0.0001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}

		p3[0] = p1[0] + dt[0]*f1/(f1-f2);
		p3[1] = p1[1] + dt[1]*f1/(f1-f2);
		p3[2] = p1[2] + dt[2]*f1/(f1-f2);

		f3 = func->EvaluateFunction(p3);

		if(fabs(f3) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		if(f3 < 0.0){
			p1[0] = p3[0];
			p1[1] = p3[1];
			p1[2] = p3[2];

			f1 = f3;
		}
		else{
			p2[0] = p3[0];
			p2[1] = p3[1];
			p2[2] = p3[2];

			f2 = f3;
		}
		dt[0] = (p2[0] - p1[0]);
		dt[1] = (p2[1] - p1[1]);
		dt[2] = (p2[2] - p1[2]);
	}

	//Bisection method
	for(j=0; j<2.0*MAX; j++){
		if(MeshData::DIST(p1, p2) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}

		p3[0] = 0.5f*(p1[0] + p2[0]);
		p3[1] = 0.5f*(p1[1] + p2[1]);
		p3[2] = 0.5f*(p1[2] + p2[2]);
		f3 = func->EvaluateFunction(p3);
		if(fabs(f3) < 0.001){
			p[0] = p3[0];
			p[1] = p3[1];
			p[2] = p3[2];
			return true;
		}
		else if(f1*f3 >= 0){
			p1[0] = p3[0];
			p1[1] = p3[1];
			p1[2] = p3[2];
			f1 = f3;
		}
		else{
			p2[0] = p3[0];
			p2[1] = p3[1];
			p2[2] = p3[2];
			f2 = f3;
		}
	}


	p[0] = start[0];
	p[1] = start[1];
	p[2] = start[2];
	return false;
}

void Smoother::moveNormalD(int iter, float dt)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;
	double (*dp)[3] = new double[vertex_N][3];
	int **link_f = mesh->vertex_link_f;
	int *degree = mesh->degree_v;

	for(int k=0; k<iter; k++){
	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		int *nei = link_f[i];
		int nei_N = degree[i];

		if(nei_N == 0){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		double total_w = 0;

		double df[3];
		df[0] = df[1] = df[2] = 0;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = i;
			int i1, i2;
			if(i0 == face[f][0]){
				i1 = face[f][1];
				i2 = face[f][2];
			}
			else if(i0 == face[f][1]){
				i1 = face[f][2];
				i2 = face[f][0];
			}
			else{
				i1 = face[f][0];
				i2 = face[f][1];
			}

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i0], vertex[i1]);
			MeshData::VEC(v2, vertex[i0], vertex[i2]);
			double c[3];
			MeshData::CROSS(c, v1, v2);

			double cx[3], cy[3], cz[3];

			cx[0] = 0;
			cx[1] = v1[2] - v2[2];
			cx[2] = v2[1] - v1[1];
			
			cy[0] = v2[2] - v1[2];
			cy[1] = 0;
			cy[2] = v1[0] - v2[0];

			cz[0] = v1[1] - v2[1];
			cz[1] = v2[0] - v1[0];
			cz[2] = 0;
	
			double m[3];
			m[0] = normal_f[f][0];
			m[1] = normal_f[f][1];
			m[2] = normal_f[f][2];
			double len = MeshData::LENGTH(m);
			double c1 = MeshData::LENGTH(c);
			if(len == 0 || c1 == 0)
				continue;

			double nm[3];
			nm[0] = c[0]/c1 - m[0]/len;
			nm[1] = c[1]/c1 - m[1]/len;
			nm[2] = c[2]/c1 - m[2]/len;

			double v[3];
			v[0] = -MeshData::DOT(cx, nm);
			v[1] = -MeshData::DOT(cy, nm);
			v[2] = -MeshData::DOT(cz, nm);
			
			double w = 1; 
			total_w += w;
	
			df[0] += w*v[0];
			df[1] += w*v[1];
			df[2] += w*v[2];
		}
		if(total_w != 0){
			dp[i][0] = df[0]/total_w;
			dp[i][1] = df[1]/total_w;
			dp[i][2] = df[2]/total_w;
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}

	for(i=0; i<vertex_N; i++){
		vertex[i][0] -= dt*(float)dp[i][0];
		vertex[i][1] -= dt*(float)dp[i][1];
		vertex[i][2] -= dt*(float)dp[i][2];
	}
	}
	delete[] dp;
}

void Smoother::moveNormalWithF(float C, vtkImplicitFunction* func)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;
	double (*dp)[3] = new double[vertex_N][3];
	int **link_f = mesh->vertex_link_f;
	int **link_v = mesh->vertex_link_v;
	int *degree_f = mesh->degree_f;
	int *degree_v = mesh->degree_v;

	for(int i=0; i<vertex_N; i++){

		if(mesh->isBound[i] == NON_MANIFOLD){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			//continue;

			int i1 = link_v[i][0];
			int i2 = link_v[i][degree_v[i]-1];

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i], vertex[i1]);
			MeshData::VEC(v2, vertex[i], vertex[i2]);

			double n[3];
			MeshData::CROSS(n, v1, v2);
			double len = MeshData::LENGTH(n);
			if((float)len < 0.001)
				continue;

			int f1 = link_f[i][0];
			int f2 = link_f[i][degree_f[i]-1];

			float n1[3], n2[3];
			n1[0] = normal_f[f1][0];
			n1[1] = normal_f[f1][1];
			n1[2] = normal_f[f1][2];

			n2[0] = normal_f[f2][0];
			n2[1] = normal_f[f2][1];
			n2[2] = normal_f[f2][2];

			double dot1 = MeshData::DOT(n1, n)/len;
			double dot2 = MeshData::DOT(n2, n)/len;

			n1[0] -= dot1*n[0]/len;
			n1[1] -= dot1*n[1]/len;
			n1[2] -= dot1*n[2]/len;
			double len1 = MeshData::LENGTH(n1);
			if((float)len1 == 0)
				continue;
			n1[0] /= len1;
			n1[1] /= len1;
			n1[2] /= len1;

			n2[0] -= dot2*n[0]/len;
			n2[1] -= dot2*n[1]/len;
			n2[2] -= dot2*n[2]/len;
			double len2 = MeshData::LENGTH(n2);
			if((float)len2 == 0)
				continue;
			n2[0] /= len2;
			n2[1] /= len2;
			n2[2] /= len2;

			dot1 = 0.5*MeshData::DOT(v1, n1);
			dot2 = 0.5*MeshData::DOT(v2, n2);

			len1 = MeshData::LENGTH(v1);
			len2 = MeshData::LENGTH(v2);
			len = len1 + len2;

			if((float)len != 0){
				dp[i][0] = (len1*dot1*n1[0] + len2*dot2*n2[0])/len;
				dp[i][1] = (len1*dot1*n1[1] + len2*dot2*n2[1])/len;
				dp[i][2] = (len1*dot1*n1[2] + len2*dot2*n2[2])/len;
			}
			continue;
		}

		int *nei, *nei_v, nei_N;
		mesh->getConsistent1Ring(i, nei_v, nei, nei_N);
		if(nei_N == 0){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		double total_w = 0;

		double pro[3];
		pro[0] = pro[1] = pro[2] = 0;

		BOOL flip_flag = false;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = face[f][0];
			int i1 = face[f][1];
			int i2 = face[f][2];

			float o[3];
			o[0] = (vertex[i0][0] + vertex[i1][0] + vertex[i2][0])/3.0;
			o[1] = (vertex[i0][1] + vertex[i1][1] + vertex[i2][1])/3.0;
			o[2] = (vertex[i0][2] + vertex[i1][2] + vertex[i2][2])/3.0;

			float d[3];
			MeshData::VEC(d, o, vertex[i]);
			double inner = MeshData::DOT(normal_f[f], d);

			float fv = func->EvaluateFunction(o);
			double w =  exp(-C*fv*fv);  //MeshData::AREA(vertex[i0], vertex[i1], vertex[i2]);

			total_w += w;

			pro[0] -= w*inner*normal_f[f][0];
			pro[1] -= w*inner*normal_f[f][1];
			pro[2] -= w*inner*normal_f[f][2];

			//float n[3];
			//float v1[3];
			//float v2[3];
			//MeshData::VEC(v1, vertex[i0], vertex[i1]);
			//MeshData::VEC(v2, vertex[i0], vertex[i2]);
			//MeshData::CROSS(n, v1, v2);
			//if(MeshData::DOT(normal_f[f], n) > 0.7){
				//flip_flag = true;
				//break;
				//TRACE("FLIP\n");
			//}
		}
		if(flip_flag){
			//mesh->laplacian(i, dp[i]);
			/*
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			for(int j=0; j<nei_N; j++){
				dp[i][0] += vertex[nei_v[j]][0] - vertex[i][0];
				dp[i][1] += vertex[nei_v[j]][1] - vertex[i][0];
				dp[i][2] += vertex[nei_v[j]][2] - vertex[i][0];
			}
			dp[i][0] /= (float)nei_N;
			dp[i][1] /= (float)nei_N;
			dp[i][2] /= (float)nei_N;*/
		}
		else if(total_w != 0){
			dp[i][0] = pro[0]/total_w;
			dp[i][1] = pro[1]/total_w;
			dp[i][2] = pro[2]/total_w;
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}

	//mesh->computeFaceNormal();
	//mesh->computeNormal();
	for(i=0; i<vertex_N; i++){
		//if(mesh->isFixed[i])
			//continue;

		//if(MeshData::DOT(dp[i], mesh->normal[i]) > 0 && !isSolidBound[i]){
			vertex[i][0] += dp[i][0];
			vertex[i][1] += dp[i][1];
			vertex[i][2] += dp[i][2];
		//}
	}
	delete[] dp;
}

void Smoother::moveVNormal(float dt)
{
	
}

void Smoother::moveNormalD2(int iter, float dt)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	float (*normal_f)[3] = mesh->normal_f;
	double (*dp)[3] = new double[vertex_N][3];
	int **link_f = mesh->vertex_link_f;
	int *degree = mesh->degree_v;

	for(int k=0; k<iter; k++){
	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i]){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		int *nei = link_f[i];
		int nei_N = degree[i];

		if(nei_N == 0){
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
			continue;
		}

		double total_w = 0;

		double df[3];
		df[0] = df[1] = df[2] = 0;

		for(int j = 0; j<nei_N; j++){
			int f = nei[j];

			int i0 = i;
			int i1, i2;
			if(i0 == face[f][0]){
				i1 = face[f][1];
				i2 = face[f][2];
			}
			else if(i0 == face[f][1]){
				i1 = face[f][2];
				i2 = face[f][0];
			}
			else{
				i1 = face[f][0];
				i2 = face[f][1];
			}

			float v1[3], v2[3];
			MeshData::VEC(v1, vertex[i0], vertex[i1]);
			MeshData::VEC(v2, vertex[i0], vertex[i2]);
			double c[3];
			MeshData::CROSS(c, v1, v2);

			double cx[3], cy[3], cz[3];

			cx[0] = 0;
			cx[1] = v1[2] - v2[2];
			cx[2] = v2[1] - v1[1];
			
			cy[0] = v2[2] - v1[2];
			cy[1] = 0;
			cy[2] = v1[0] - v2[0];

			cz[0] = v1[1] - v2[1];
			cz[1] = v2[0] - v1[0];
			cz[2] = 0;
	
			double m[3];
			m[0] = normal_f[f][0];
			m[1] = normal_f[f][1];
			m[2] = normal_f[f][2];

			double c1 = MeshData::LENGTH(c);
			if(c1 == 0)
				continue;
			double nm[3];
			nm[0] = c[0]/c1 - m[0];
			nm[1] = c[1]/c1 - m[1];
			nm[2] = c[2]/c1 - m[2];

			double v[3];
			v[0] = -MeshData::DOT(cx, nm);
			v[1] = -MeshData::DOT(cy, nm);
			v[2] = -MeshData::DOT(cz, nm);

			float n[3];
			n[0] = c[0]/c1;
			n[1] = c[1]/c1;
			n[2] = c[2]/c1;

			double en = 1.0 - MeshData::DOT(n, m);
			v[0] += -MeshData::DOT(n, cx)*en;
			v[1] += -MeshData::DOT(n, cy)*en;
			v[2] += -MeshData::DOT(n, cz)*en;

			double w = 1; 
			total_w += w;
	
			df[0] += w*v[0];
			df[1] += w*v[1];
			df[2] += w*v[2];
		}
		if(total_w != 0){
			dp[i][0] = df[0]/total_w;
			dp[i][1] = df[1]/total_w;
			dp[i][2] = df[2]/total_w;
		}
		else
			dp[i][0] = dp[i][1] = dp[i][2] = 0;
	}

	for(i=0; i<vertex_N; i++){
		vertex[i][0] -= dt*(float)dp[i][0];
		vertex[i][1] -= dt*(float)dp[i][1];
		vertex[i][2] -= dt*(float)dp[i][2];
	}
	}
	delete[] dp;
}

float Smoother::computeMaxVertexErr(vtkImplicitFunction *func)
{
	int vertex_N = mesh->vertex_N;
	float (*vertex)[3] = mesh->vertex;
	//float total_area = 0;
	float max = 0;
	for(int i=0; i<vertex_N; i++){
		if(mesh->isBound[i])
			continue;
		/*
		int *nei, nei_N;
		float area = 0;
		mesh->getConsistent1Ring(i, nei, nei_N);
		for(int j=0; j<nei_N; j++)
			area += MeshData::AREA(vertex[i], vertex[nei[j]], vertex[nei[(j+1)%nei_N]]);
			*/
		float f = fabs(func->EvaluateFunction(vertex[i]));
		float g[3];
		func->EvaluateGradient(vertex[i], g);
		double len = MeshData::LENGTH(g);
		//if((float)len == 0)
		//	continue;

		if(max < f/len)
			max = f/len;
	}
	return max;
}

float Smoother::computeMaxFaceNormalErr(vtkImplicitFunction *func)
{
	double max = 0;
	int face_N = mesh->face_N;
	float (*vertex)[3] = mesh->vertex;
	int (*face)[3] = mesh->face;
	BOOL *isBound = mesh->isBound;
	float total_area = 0;

	for(int i=0; i<face_N; i++){
		int* fi = face[i];
		//if(isBound[fi[0]] || isBound[fi[1]] || isBound[fi[2]])
			//continue;

		float c[3];
		c[0] = (vertex[fi[0]][0] + vertex[fi[1]][0] + vertex[fi[2]][0])/3.0f;
		c[1] = (vertex[fi[0]][1] + vertex[fi[1]][1] + vertex[fi[2]][1])/3.0f;
		c[2] = (vertex[fi[0]][2] + vertex[fi[1]][2] + vertex[fi[2]][2])/3.0f;
		float g[3];
		func->EvaluateGradient(c, g);
		float len1 = MeshData::LENGTH(g);

		float v1[3], v2[3], n[3];
		MeshData::VEC(v1, vertex[fi[0]], vertex[fi[1]]);
		MeshData::VEC(v2, vertex[fi[0]], vertex[fi[2]]);
		MeshData::CROSS(n, v1, v2);
		float len2 = MeshData::LENGTH(n);
		
		if(len1*len2 == 0)
			continue;

		double dif[3];
		dif[0] = n[0]/len2 + g[0]/len1;
		dif[1] = n[1]/len2 + g[1]/len1;
		dif[2] = n[2]/len2 + g[2]/len1;
		double err = MeshData::LENGTH(dif);

		if(max < err)
			max = err;
	
	}
	return max;
}

void Smoother::correctImplicitSurfaceNew(vtkImplicitFunction *func, int iter, int off_iter, BOOL err)
{
	int i;

	int from = 0;
	FILE *v_file, *f_file;
	float c;
	if(err){	
		v_file = fopen("vertex_err.dat","w");
		fprintf(v_file, "{");
		float max = 0;
		for(int i=0; i<mesh->vertex_N; i++){
			float f = func->EvaluateFunction(mesh->vertex[i]);
			if(max < f*f)
				max = f*f;
		}
		//c = 1.0f/max;
		c = 1.0;
		fprintf(v_file, "%f", c*computeVertexErr(func));

		f_file = fopen("face_err.dat","w");
		fprintf(f_file, "{");
		fprintf(f_file, "%f", computeFaceNormalErr(func));
	}

	for(i=0; i<iter; i++){
		setNormalAsGrad(func);
		moveNormal(from);
		mesh->computeNormal();
		tangentLap(0.1, from);
		keepOnSurface(func, from);

		if(err){
			fprintf(v_file, ",%f", c*computeVertexErr(func));
			fprintf(f_file, ",%f", computeFaceNormalErr(func));
		}
	}
	for(i=0; i<off_iter; i++){
		setNormalAsGradWithoutNormalize(func);
		moveNormalD(5, 0.2);
		//setNormalAsGradWithoutNormalize(func);
		keepOnSurfaceT(func, 0.01f);

		if(err){
			fprintf(v_file, ",%f", c*computeVertexErr(func));
			fprintf(f_file, ",%f", computeFaceNormalErr(func));
		}
	}

	if(err){
		fprintf(v_file, "}");
		fprintf(f_file, "}");
		fclose(v_file);
		fclose(f_file);
	}
}

void Smoother::setNormalAsGradWithoutNormalize(vtkImplicitFunction *func)
{
	int face_N = mesh->face_N;
	int (*face)[3] = mesh->face;
	float (*vertex)[3] = mesh->vertex;
	float (*normal)[3] = mesh->normal_f;
	if(normal == NULL)
		normal = mesh->normal_f = new float[face_N][3];

	for(int i=0; i<face_N; i++){
		int* fi = face[i];
		if(fi[0] < 0)
			continue;
		float c[3];
		c[0] = (vertex[fi[0]][0] + vertex[fi[1]][0] + vertex[fi[2]][0])/3.0f;
		c[1] = (vertex[fi[0]][1] + vertex[fi[1]][1] + vertex[fi[2]][1])/3.0f;
		c[2] = (vertex[fi[0]][2] + vertex[fi[1]][2] + vertex[fi[2]][2])/3.0f;
		
		float g[3];
		func->EvaluateGradient(c, g);
		normal[i][0] = -g[0];
		normal[i][1] = -g[1];
		normal[i][2] = -g[2];
	}
}
